
class GUI{
    constructor(game){
        let that = this;
        this.ui = [];
        this.game = game;
        ///При изменениии сцены убивает гуи
        this.game.state.onShutDownCallback = function(){
            that.delete();
        }
    }
    update(){
        for(let i =0 ;i < this.ui.length ;i++){
            ///Если UI отмечен для удаления
            if(this.ui[i].isDeleted()){
                this.ui.splice(i,1);
                i--;
            }else
                this.ui[i].update();
        }
    }
    getNewInputPriority(){
        let max = 0;
       for(let i in this.ui){
            let maxInputOfChilds = this.ui[i].getMaxInputPriority();
            if(maxInputOfChilds > max)
                max = maxInputOfChilds;
        }
        return max + 1; 
    }
    delete(){
        for(let i =0 ;i < this.ui.length ;i++){
            this.ui[i].delete();
            this.ui.splice(i,1);
            i--;
        }
        this.ui.splice(0,this.ui.length);
    }
    addUI(x,y,w,h,layerGroup,inputPriority,layersCount){
        if (inputPriority === null){
            inputPriority = this.getNewInputPriority();
        }
        let num = this.ui.push( new UI(x,y,w,h,layerGroup,this.game,inputPriority,layersCount,false,this,null) );///Создаем новый UI
        return this.ui[num -1];
    }
    
}
///Класс Интерфейс  
class UI{
    /*
    *******Создание объекта интерфейса********
    x,y - координаты(int)
    w,h - размеры(int)
    layerGroup - слой (группа фэйзера)(obj)
    game - объект фэйзера (obj)
    inputPriority - приоритет input (чем больше тем более приоритет (int))
    layersCount - количество слоев в этом объекте интерфейса(int)
    isFixed - параметр определяющий зафиксирована ли UI относительно камеры+(int)
    */
    constructor(x,y,w,h,layerGroup,game,inputPriority,layersCount,fixedToCamera,GUI,parentUI){
        let that = this;
        this.game = game;
        this.GUI = GUI;
        this.parentUI = parentUI;
        this.x = x;
        this.y = y;
        this.lastX = this.x;
        this.lastY = this.y;
        this.currentX = this.x;
        this.currentY = this.y;
        this.height = h;
        this.width = w;
        this.xSize = null;
        this.ySize =  null;
        this.midX = this.x + (this.width/2);
        this.midY = this.y + (this.height/2);
        this.inputPriority = inputPriority;
        this.layerGroup = layerGroup;
        this.textFill = "#FAFAD2";
        if (this.layerGroup === null){
            this.layerGroup = this.game.add.group();
        }
        ///Количество слоев
        this.layersCount =layersCount;
        ///Группа слоев
        ///Массив слоев
        this.layers = [];
        for(let i = 0; i< this.layersCount; i++){
            this.layers[i] = this.game.add.group();
            if(this.layerGroup){
                this.layerGroup.add(this.layers[i]);
            }else{
                console.log('GUI: ERROR:UNKNOWN LAYER!')
            }
            
            
        }
        //console.log(this.inputPriority)
        this.onDelete = [];
        this.onUpdate = [];
        this.onCreate = [];
        this.backgrounds = [];
        this.images = [];
        this.graphics = [];
        this.buttons = [];
        this.sliders = [];
        this.texts =[];
        this.audio =[];
        this.inpuFields = [];
        this.UIArray = [];
        this.domElements = [];
        this.deleted = false;
        this.sortByY = false;
        this.fixedToCamera = fixedToCamera || false;
        
        if(parentUI !== null){
            let lastUIX = that.parentUI.x;
            let lastUIY = that.parentUI.y;
            
            this.addUpdate(function(){
                if((that.parentUI.x !== lastUIX) || (that.parentUI.y !== lastUIY)){
                    let diffX = that.parentUI.x - lastUIX;
                    let diffY = that.parentUI.y - lastUIY;
                    that.x += diffX;
                    that.y += diffY;

                    lastUIX = that.parentUI.x;
                    lastUIY = that.parentUI.y;
                    //diffX = that.x - that.parentUI.x;
                    //diffY = that.y - that.parentUI.y;
                }
            })
            
        }
        
    }
    addUpdate(func){
        this.onUpdate.push(func);
    }
    addDelete(func){
        this.onDelete.push(func);
    }
    getMaxInputPriority(){
        let max = this.inputPriority;
        for(let i in this.UIArray){
            let maxInputOfChilds = this.UIArray[i].getMaxInputPriority();
            if(maxInputOfChilds > max)
                max = maxInputOfChilds;
        }
        return max;
    }
    setTextFill(fill){
        this.textFill = fill;
    }
    /*
    *******Добавить Элементу Интерфейса Элемент интерфейса********
    layer - номер слоя (int)
    relativCoord - координаты относительно текущего элемента (true/false)
    x,y - координаты(int)
    w,h - размеры(int)
    layersCount - количество слоев в этом объекте интерфейса(int)
    */
    addUI(layer,relativCoord,x,y,w,h,layersCount){
       
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        let num = this.UIArray.push(new UI(coordX,coordY,w,h,this.layers[layer],this.game,this.inputPriority +layer,layersCount,this.fixedToCamera,this.GUI,this));
        return this.UIArray[num - 1];
    }
    
    /*
    *******Добавить Фон Картинкой Элементу интерфейса********
    layer - номер слоя (int)
    imageKey - ключ картинки(string)
    alpha - прозрачность (от 0 до 1)
    inputEnable - возможность input (true,false)
    */
    addBackground(layer,imageKey,alpha,inputEnable){
        let that = this;
       let num = this.images.push(this.layers[layer].create(this.x,this.y,imageKey));
        let image = this.images[num -1];
        if(inputEnable){
            image.inputEnabled = true;
            image.input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
            
        image.height = this.height;
        image.width = this.width;
        image.alpha = alpha;
        image.fixedToCamera = this.fixedToCamera;
        ///Приоритет ввода/нажатия  = приоритет UI + номер слоя
        let lastUIX = this.x;
        let lastUIY = this.y;
        let lastUIWidth = this.width;
        let lastUIHeight = this.height;   
        this.addUpdate(function(){
            if(((that.x !== lastUIX) || (that.y !== lastUIY)) || ((that.width !== lastUIWidth) || (that.height !== lastUIHeight))) {
                image.x = that.x;
                image.y = that.y;
                image.width = that.width;
                image.height = that.height;
                
                lastUIX = that.x;
                lastUIY = that.y;
                lastUIWidth = that.width;
                lastUIHeight = that.height;
            }
        })
        return image;
    }
    addCastomSizeBackground(layer,w,h,imageKey,alpha,inputEnable){
        
       let num = this.backgrounds.push(this.layers[layer].create((this.x + this.width/2) - (w/2),(this.y + this.height/2) - (h/2),imageKey));
        if(inputEnable){
            this.backgrounds[num -1].inputEnabled = true;
            this.backgrounds[num -1].input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
            
        this.backgrounds[num -1].height = h;
        this.backgrounds[num -1].width = w;
        this.backgrounds[num -1].alpha = alpha;
        this.backgrounds[num -1].fixedToCamera = this.fixedToCamera;
        ///Приоритет ввода/нажатия  = приоритет UI + номер слоя
        return this.backgrounds[num -1];
    }
    /*
    *******Добавить Фон Квадратом Элементу интерфейса********
    layer - номер слоя (int)
    isFill - закрашен ли квадрат (true/false)
    isBorder - есть ли рамка(true/false)
    fillColor - цвет заливки (пример 0x000000)
    fillBorder - цвет рамки (пример 0x000000)
    sizeBorder -размер рамки (int)
    alphaFill - прозрачность заливки (от 0 до 1)
    alphaBorder - прозрачность рамки (от 0 до 1)
    inputEnable - возможность input (true,false)
    */
    addBackgroundRect(layer,isFill,isBorder,fillColor,fillBorder,sizeBorder,alphaFill,alphaBorder,inputEnable){
        let that = this;
        let num = this.graphics.push(this.game.add.graphics(0,0));
        let grath = this.graphics[num -1];
        function addBgRect(x,y,width,height){
            if(isBorder){
                grath.lineStyle(sizeBorder, fillBorder, alphaBorder);
            }
            if(isFill){
                grath.beginFill(fillColor);

            }
            let rect = grath.drawRect(x,y,width,height);
            if(isFill){
                grath.endFill();
            }
            if(inputEnable){
                grath.inputEnabled = true;

                grath.input.priorityID = that.inputPriority + (layer/that.layersCount);

            }

            grath.alpha = alphaFill;
            
        }
        this.layers[layer].add(grath);
        grath.fixedToCamera = this.fixedToCamera;
        let oldUiX = this.x;
        let oldUiY = this.y;
        let oldUiWidth = this.width;
        let oldUiHeight = this.height;
        this.addUpdate(function(){
            
            if( (oldUiX !== that.x)||(oldUiY !== that.y)||(oldUiWidth !== that.width)||(oldUiHeight !== that.height) ){
                grath.clear();
                addBgRect(that.x,that.y,that.width,that.height);
                oldUiX = that.x;
                oldUiY = that.y;
                oldUiWidth = that.width;
                oldUiHeight = that.height;
            }
        });
        addBgRect(this.x,this.y,this.width,this.height);
        return grath;
        
    }
    addBackgroundRoundedRect(layer,isFill,isBorder,fillColor,fillBorder,sizeBorder,alphaFill,alphaBorder,radius,inputEnable){
        let that = this;
        let num = this.graphics.push(this.game.add.graphics(0,0));
        let grath = this.graphics[num -1];
        function addBgRect(x,y,width,height){
            if(isBorder){
                grath.lineStyle(sizeBorder, fillBorder, alphaBorder);
            }
            if(isFill){
                grath.beginFill(fillColor);

            }
            let rect = grath.drawRoundedRect(x,y,width,height,radius);
            if(isFill){
                grath.endFill();
            }
            if(inputEnable){
                grath.inputEnabled = true;

                grath.input.priorityID = that.inputPriority + (layer/that.layersCount);

            }

            
            
        }
        grath.alpha = alphaFill;
        this.layers[layer].add(grath);
        grath.fixedToCamera = this.fixedToCamera;
        let oldUiX = this.x;
        let oldUiY = this.y;
        let oldUiWidth = this.width;
        let oldUiHeight = this.height;
        this.addUpdate(function(){
            
            if( (oldUiX !== that.x)||(oldUiY !== that.y)||(oldUiWidth !== that.width)||(oldUiHeight !== that.height) ){
                grath.clear();
                addBgRect(that.x,that.y,that.width,that.height);
                oldUiX = that.x;
                oldUiY = that.y;
                oldUiWidth = that.width;
                oldUiHeight = that.height;
            }
            
        });
        
        addBgRect(this.x,this.y,this.width,this.height);
        return grath;
    
    }
    addFullWindowRect(layer,x,y,w,h,isFill,fillColor,alphaFill,onClick){
        
        let num = this.graphics.push(this.game.add.graphics(0,0));
        if(isFill){
            this.graphics[num -1].beginFill(fillColor);
        }
        this.graphics[num -1].drawRect(x,y,w,h);
        if(isFill){
            this.graphics[num -1].endFill();
        }
        this.graphics[num -1].alpha = alphaFill;
        
        this.graphics[num -1].inputEnabled = true;
        this.graphics[num -1].input.priorityID = this.inputPriority + (layer/this.layersCount);
        this.graphics[num -1].events.onInputDown.add(function(){
            setTimeout(function(){
                onClick();
            },50);
        });
        this.layers[layer].add(this.graphics[num -1]);
        this.graphics[num -1].fixedToCamera = this.fixedToCamera;
        return this.graphics[num -1];
    }
    addFullWindowBackRect(layer,x,y,w,h,isFill,fillColor,alphaFill){
        
        let num = this.graphics.push(this.game.add.graphics(0,0));
        if(isFill){
            this.graphics[num -1].beginFill(fillColor);
        }
        this.graphics[num -1].drawRect(x,y,w,h);
        if(isFill){
            this.graphics[num -1].endFill();
        }
        this.graphics[num -1].alpha = alphaFill;
        this.layers[layer].add(this.graphics[num -1]);
        this.graphics[num -1].fixedToCamera = this.fixedToCamera;
        return this.graphics[num -1];
    }
    /*
    *******Добавить Кнопку Элементу интерфейса********
    layer - номер слоя (int)
    relativCoord - координаты относительно текущего элемента (true/false)
    x,y - координаты(int)
    w,h - размеры(int)
    imageKey - ключ картинки(string)
    onClick - функция которая вызывается при клике (function)
    */
    addButton(layer,relativCoord,x,y,w,h,imageKey,onClick,onDownSoundKey,onOverSoundKey,volume,alpha){
        let that = this;
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        
        let num = this.images.push(this.layers[layer].add(new Phaser.Button(this.game,coordX,coordY,imageKey,function(){
            setTimeout(function(){
                onClick();
            },50);
                
                
            },this,1,0,2,1))); 
        let button = this.images[num - 1];
        button.alpha = alpha;
        let onDownSound = this.addSound(onDownSoundKey,false,volume);
        let onOverSound = this.addSound(onOverSoundKey,false,volume);
        button.events.onInputDown.add(function(){
            onDownSound.play();
        }, that);
         button.events.onInputOver.add(function(){
            onOverSound.play();
        }, that);
        button.anchor.setTo(0.5, 0.5);
        button.height = h;
        button.width = w;
        button.input.priorityID = this.inputPriority + (layer/this.layersCount);
        button.fixedToCamera = this.fixedToCamera;
        
        let lastUIX = this.x;
        let lastUIY = this.y;
        let lastInputState = button.input.enabled;
        function onInputEnableChange(){
            if(!button.input.enabled){
                button.frame = 3;
            }else{
                button.frame = 0;
            }
        }
        this.addUpdate(function(){
            if(lastInputState !== button.input.enabled){
                onInputEnableChange();
                lastInputState = button.input.enabled;
            }
            if((that.x !== lastUIX) || (that.y !== lastUIY)){
                let diffX = that.x - lastUIX;
                let diffY = that.y - lastUIY;
                button.x += diffX;
                button.y += diffY;

                lastUIX = that.x;
                lastUIY = that.y;
                //diffX = that.x - that.parentUI.x;
                //diffY = that.y - that.parentUI.y;
            }
        });
        return button;
    }
    addButtonWithText(layer,relativCoord,x,y,w,h,imageKey,onClick,onDownSoundKey,onOverSoundKey,volume,string,style,alpha){
        let that = this;
        let button = this.addButton(layer,relativCoord,x,y,w,h,imageKey,onClick,onDownSoundKey,onOverSoundKey,volume);
   
        button.alpha = alpha;
        let text = this.addText(layer,false,button.x,button.y,string,style,1,false);
        text.wordWrap = true;
        text.wordWrapWidth = w*0.9;
        text.anchor.setTo(0.5,0.5);
        text.alpha = alpha;
        //text.x = Math.round(button.x - (text.width)/2);
        //text.y = Math.round(button.y - (text.height)/2);
        text.lineSpacing = -15;
    
        return {button:button,text:text};
    }
    addButtonWithBitMapText(layer,relativCoord,x,y,w,h,imageKey,onClick,onDownSoundKey,onOverSoundKey,volume,string,font,size,alpha){
        let that = this;
        let button = this.addButton(layer,relativCoord,x,y,w,h,imageKey,onClick,onDownSoundKey,onOverSoundKey,volume);
        //style.wordWrap = true;
        //style.wordWrapWidth = w*0.9;
        button.alpha = alpha;
        let text = this.addBitMapText(layer,false,button.x,button.y,string,font,size,alpha,false)
        text.anchor.setTo(0.5,0.5);
        text.alpha = alpha;
        //text.x = Math.round(button.x - (text.width)/2);
        //text.y = Math.round(button.y - (text.height)/2);
        text.lineSpacing = -15;
    
        return {button:button,text:text};
    }
    addLongClickButton(layer,relativCoord,x,y,w,h,imageKey,onClick,onLongClick){
        let that = this;
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        let clamp = false;
        let progress = 1;
        let clickStart = null;
        let num = this.buttons.push(this.addSprite(layer,relativCoord,x,y,w,h,imageKey,1,true));
        let button = this.buttons[num - 1];
        button.events.onInputDown.add(function(){
            clickStart = new Date();
            onClick();
            button.frame = 2;
        }, that);
         button.events.onInputUp.add(function(){
             clickStart = null;
             clamp = false;
             button.frame = 1;
        }, that);
         button.events.onInputOver.add(function(){
             button.frame = 1;
        }, that);
         button.events.onInputOut.add(function(){
             clickStart = null;
             clamp = false;
             button.frame = 0;
        }, that);
        button.input.useHandCursor = true;
        button.anchor.setTo(0.5, 0.5);
        button.height = h;
        button.width = w;
        button.input.priorityID = this.inputPriority + (layer/this.layersCount);
        
        let lastUIX = this.x;
        let lastUIY = this.y;
            
        this.addUpdate(function(){
            if((that.x !== lastUIX) || (that.y !== lastUIY)){
                let diffX = that.x - lastUIX;
                let diffY = that.y - lastUIY;
                button.x += diffX;
                button.y += diffY;

                lastUIX = that.x;
                lastUIY = that.y;
            }
            if(clickStart){
                if((new Date - clickStart) > 500){
                    progress *= 1.005;
                    if(progress >= 3){
                        progress = 3;
                    }
                    onLongClick(progress);
                }else{
                    progress = 1;
                }
                
            }
        });
      
        button.fixedToCamera = this.fixedToCamera;
        return button;
    }
    addButtonToggle(layer,relativCoord,x,y,w,h,imageKey,originState,getStateFunction,onClick){
        let that = this;
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        let state = originState;
        let num = this.buttons.push(this.addSprite(layer,relativCoord,x,y,w,h,imageKey,1,true));
        this.buttons[num - 1].events.onInputDown.add(function(){
            state = !state;
            onClick();
        }, that);
        this.buttons[num - 1].input.useHandCursor = true;
        this.buttons[num - 1].anchor.setTo(0.5, 0.5);
        this.buttons[num - 1].height = h;
        this.buttons[num - 1].width = w;
        this.buttons[num - 1].input.priorityID = this.inputPriority + (layer/this.layersCount);
        
        this.onUpdate.push(function(){
            state = getStateFunction();
            if(state){
                that.buttons[num - 1].frame = 1;
            }else{
                that.buttons[num - 1].frame = 0;
            }
        });
        this.buttons[num - 1].fixedToCamera = this.fixedToCamera;
        return this.buttons[num - 1];
    }
    /*
    *******Добавить Прямоугольную Кнопку Элементу интерфейса********
    layer - номер слоя (int)
    relativCoord - координаты относительно текущего элемента (true/false)
    x,y - координаты(int)
    w,h - размеры(int)
    imageKey - ключ картинки(string)
    onClick - функция которая вызывается при клике (function)
    */
    addRectButton(layer,relativCoord,x,y,w,h,isBorder,fillColor,fillBorder,sizeBorder,alphaFill,alphaBorder,text,sizeText,alphaText,onOver,onOut,onClick){
        ///Относительные ли UI координаты
        let coordX,coordY,grathObj,textObj,num;
        if(relativCoord){
            coordX = this.x+(x - (w/2));
            coordY = this.y+(y - (h/2));
        }
        else{
            coordX = (x - (w/2));
            coordY = (y - (h/2));
        }
        
        
        
        num = this.graphics.push(this.game.add.graphics(0,0));
        grathObj = this.graphics[num -1];
        if(isBorder){
            grathObj.lineStyle(sizeBorder, fillBorder, alphaBorder);
        }
        grathObj.fixedToCamera = this.fixedToCamera;
        grathObj.beginFill(fillColor);
        
        grathObj.drawRect(coordX,coordY,w,h);
        
        grathObj.endFill();
        
        grathObj.alpha = alphaFill;
        grathObj.inputEnabled = true;
        grathObj.input.useHandCursor = true;
        grathObj.input.priorityID = this.inputPriority + (layer/this.layersCount);
        grathObj.events.onInputDown.add(function(){
            setTimeout(function(){
                onClick();
            },50);
        });
        grathObj.events.onInputOver.add(function(){
            onOver();
        });
        grathObj.events.onInputOut.add(function(){
            onOut();
        });
        
        
        
        this.layers[layer].add(grathObj);
        
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        
        num = this.texts.push(this.game.add.text(coordX,coordY,text,{ font: ""+sizeText+"px", fill: "#000000",maxLines: 15,wordWrap: true, wordWrapWidth: this.width, align: "left"},this.layers[layer]));
        textObj = this.texts[num - 1];
        textObj.anchor.setTo(0.5, 0.5);
        textObj.alpha = alphaText;
        textObj.fixedToCamera = this.fixedToCamera;
        //textObj.font = 'Arial';#4A90E2
        textObj.fill = "#C0C0C0";
        //textObj.fontWeight = 'bold';
        textObj.stroke = '#000000';
        textObj.strokeThickness = 5;  
        
        grathObj.events.onInputOver.add(function(){
            textObj.fill = "#FA8072"
        });
        grathObj.events.onInputOut.add(function(){
           textObj.fill = "#C0C0C0";
        });
        
        
        return grathObj;
    }
    /*
    *******Добавить Поле Ввода Элементу интерфейса********
    layer - номер слоя (int)
    relativCoord - координаты относительно текущего элемента (true/false)
    x,y - координаты(int)
    w,h - размеры(int)
    secret - секретное ли поле (пример пароль) (true/false)
    */
    addInputField(layer,relativCoord,x,y,w,h,secret,size,placeHolder){
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+(x - (w/2));
            coordY = this.y+(y - (h/2));
        }
        else{
            coordX = (x - (w/2));
            coordY = (y - (h/2));
        }
        
        let num;
        if(secret){
           num = this.inpuFields.push(game.add.inputField(coordX,coordY, {
                font: ""+size+"px Arial",
                fill: '#000000',
                fillAlpha: 0,
                fontWeight: 'normal',
                width: w,
                height: h,
                max: 20,
                padding: 8,
                borderWidth: 1,
                borderColor: '#000',
                borderRadius: 6,
                placeHolder: placeHolder,
                type: PhaserInput.InputType.password,
                textAlign: 'center',
                //zoom: true
            }));
        }
        else{
            num = this.inpuFields.push(game.add.inputField(coordX,coordY, {
                font: ""+size+"px Arial",
                fill: '#000000',
                fillAlpha: 0,
                fontWeight: 'normal',
                width: w,
                height: h,
                max: 20,
                padding: 8,
                borderWidth: 1,
                borderColor: '#000',
                borderRadius: 6,
                placeHolder: placeHolder,
                textAlign: 'center',
                //zoom: true
            }));
        }
        this.layers[layer].add(this.inpuFields[num - 1]);
        return this.inpuFields[num - 1];
    }
    /*
    *******Добавить Текст Элементу интерфейса********
    layer - номер слоя (int)
    relativCoord - координаты относительно текущего элемента (true/false)
    x,y - координаты(int)
    text - сам текст (string)
    size - размер шрифта (int)
    alpha - прозрачность (от 0 до 1)
    inputEnable - возможность input (true,false)
    */
    addText(layer,relativCoord,x,y,text,style,alpha,inputEnable){
        let that = this;
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        
        let num = this.texts.push(this.game.add.text(coordX,coordY,text,style,this.layers[layer]));
        
        let textSprite = this.texts[num - 1];
        if(style.wordWrap){
            if(!style.wordWrapWidth){
                textSprite.wordWrapWidth =  this.width*0.9;
                
            }
        }
        textSprite.alpha = alpha;
        
        if(inputEnable){
            textSprite.inputEnabled = true;
            textSprite.input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
        textSprite.fixedToCamera = this.fixedToCamera;
        textSprite.anchor.setTo(0.5,0.5);
        let lastUIX = this.x;
        let lastUIY = this.y;
            
        this.addUpdate(function(){
            if((that.x !== lastUIX) || (that.y !== lastUIY)){
                let diffX = that.x - lastUIX;
                let diffY = that.y - lastUIY;
                textSprite.x += diffX;
                textSprite.y += diffY;

                lastUIX = that.x;
                lastUIY = that.y;
            }
        })
        return textSprite;
    }
    addBitMapText(layer,relativCoord,x,y,text,font,size,alpha,inputEnable){
        let that = this;
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        
        //assets.addTextStyle('commonCenter',{font: "TestFont",size :28,isBold:false,  fill: "#000000",maxLines: 15,wordWrap: true, align: "center",stroke : '#000000',strokeThickness : 0, lineSpacing : -10});
        /*
        let data = {
            font : ""+style.size+"px",
            fill : style.fill,
            maxLines: style.maxLines,
            wordWrap: style.wordWrap,
            align:style.align
        }
        */
        //add.bitmapText(mainUI.x + mainUI.width*0.05,mainUI.y + mainUI.width*0.05, 'test','Выберите расу персонажа:',64);
       // let num = this.texts.push(this.game.add.bitmapText(coordX,coordY,font,text,64));
        let num = this.texts.push(this.game.add.bitmapText(coordX,coordY,font,text,size));
        let textSprite = this.texts[num - 1];
        this.layers[layer].add(textSprite);
        textSprite.alpha = alpha;
        
        if(inputEnable){
            textSprite.inputEnabled = true;
            textSprite.input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
        textSprite.fixedToCamera = this.fixedToCamera;
        textSprite.anchor.setTo(0.5,0.5);
        let lastUIX = this.x;
        let lastUIY = this.y;
            
        this.addUpdate(function(){
            if((that.x !== lastUIX) || (that.y !== lastUIY)){
                let diffX = that.x - lastUIX;
                let diffY = that.y - lastUIY;
                textSprite.x += diffX;
                textSprite.y += diffY;

                lastUIX = that.x;
                lastUIY = that.y;
            }
        })
        return textSprite;
    }
    
    addBitMapTextButton(layer,relativCoord,x,y,text,font,size,alpha,onOver,onOut,onClick){
        let that = this;
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        let num = this.texts.push(this.game.add.bitmapText(coordX,coordY,font,text,size));
        let textSprite = this.texts[num - 1];
        this.layers[layer].add(textSprite);
        textSprite.alpha = alpha;
        textSprite.inputEnabled = true;
        textSprite.input.useHandCursor = true;
        textSprite.input.priorityID = this.inputPriority + (layer/this.layersCount);
        textSprite.fixedToCamera = this.fixedToCamera;
        textSprite.anchor.setTo(0.5,0.5);
        textSprite.events.onInputDown.add(function(){
            setTimeout(function(){
                onClick();
            },50);
        });
        textSprite.events.onInputOver.add(function(){
            onOver();
        });
        textSprite.events.onInputOut.add(function(){
            onOut();
        });
        let lastUIX = this.x;
        let lastUIY = this.y;
            
        this.addUpdate(function(){
            if((that.x !== lastUIX) || (that.y !== lastUIY)){
                let diffX = that.x - lastUIX;
                let diffY = that.y - lastUIY;
                textSprite.x += diffX;
                textSprite.y += diffY;

                lastUIX = that.x;
                lastUIY = that.y;
            }
        })
        return textSprite;
    }
    /*
    *******Получить слой. Точнее вернуть слой и приоритет input  для последоющего использования в объекте********
    layer - номер слоя (int)
    callback - функция в которую передаются слой и приоритет клика
    */
    addLine(layer,relativCoord,x1,y1,x2,y2,fillColor,size,alpha,inputEnable){
        ///Относительные ли UI координаты
        let coordX1,coordY1,coordX2,coordY2;
        if(relativCoord){
            coordX1 = this.x+x1;
            coordY1 = this.y+y1;
            coordX2 = this.x+x2;
            coordY2 = this.y+y2;
        }
        else{
            coordX1 = x1;
            coordY1 = y1;
            coordX2 = x2;
            coordY2 = y2;
        }
     
        let num = this.graphics.push(this.game.add.graphics(0,0));

        this.graphics[num -1].lineStyle(size, fillColor, 1);
        
        this.graphics[num -1].beginFill(fillColor);
        this.graphics[num -1].moveTo(coordX1,coordY1);
        this.graphics[num -1].lineTo(coordX2, coordY2);
        this.graphics[num -1].endFill();
       
        this.graphics[num -1].alpha = alpha;
        if(inputEnable){
            this.graphics[num -1].inputEnabled = true;
            this.graphics[num -1].input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
        this.layers[layer].add(this.graphics[num -1]);
        this.graphics[num -1].fixedToCamera = this.fixedToCamera;
        return this.graphics[num -1];
    }
    changeLayer(layer,object){
        this.layers[layer].add(object);
    }
    getLayer(layer,callback){
        
        callback(this.layers[layer],this.inputPriority);
    }
    /*
    *******Добавить Квадрат Элементу интерфейса********
    layer - номер слоя (int)
    relativCoord - координаты относительно текущего элемента (true/false)
    x,y - координаты(int)
    w,h - размеры(int)
    isFill - закрашен ли квадрат (true/false)
    isBorder - есть ли рамка(true/false)
    fillColor - цвет заливки (пример 0x000000)
    fillBorder - цвет рамки (пример 0x000000)
    sizeBorder -размер рамки (int)
    alphaFill - прозрачность заливки (от 0 до 1)
    alphaBorder - прозрачность рамки (от 0 до 1)
    inputEnable - возможность input (true,false)
    */
    addRect(layer,relativCoord,x,y,w,h,isFill,isBorder,fillColor,fillBorder,sizeBorder,alphaFill,alphaBorder,inputEnable){
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+(x - (w/2));
            coordY = this.y+(y - (h/2));
        }
        else{
            coordX = (x - (w/2));
            coordY = (y - (h/2));
        }
        
        let num = this.graphics.push(this.game.add.graphics(0,0));
        if(isBorder){
            this.graphics[num -1].lineStyle(sizeBorder, fillBorder, alphaBorder);
        }
        if(isFill){
            this.graphics[num -1].beginFill(fillColor);
        }
        this.graphics[num -1].drawRect(coordX,coordY,w,h);
        if(isFill){
            this.graphics[num -1].endFill();
        }
        this.graphics[num -1].alpha = alphaFill;
        if(inputEnable){
            this.graphics[num -1].inputEnabled = true;
            this.graphics[num -1].input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
        this.layers[layer].add(this.graphics[num -1]);
        this.graphics[num -1].fixedToCamera = this.fixedToCamera;
        return this.graphics[num -1];
    }
    updateRect(graphics,relativCoord,x,y,w,h,isFill,isBorder,fillColor,fillBorder,sizeBorder,alphaFill,alphaBorder,inputEnable){
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+(x - (w/2));
            coordY = this.y+(y - (h/2));
        }
        else{
            coordX = (x - (w/2));
            coordY = (y - (h/2));
        }
        graphics.clear();
        if(isBorder){
            graphics.lineStyle(sizeBorder, fillBorder, alphaBorder);
        }
        if(isFill){
            graphics.beginFill(fillColor);
        }
        graphics.drawRect(coordX,coordY,w,h);
        if(isFill){
            graphics.endFill();
        }
        graphics.alpha = alphaFill;
        if(inputEnable){
            graphics.inputEnabled = true;
            graphics.input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
        graphics.fixedToCamera = this.fixedToCamera;
        return graphics;
    }
    ///newRectFunc - должен возвращать новые координаты квадрата 
    addUpdatedRect(layer,relativCoord,x,y,w,h,isFill,isBorder,fillColor,fillBorder,sizeBorder,alphaFill,alphaBorder,inputEnable,newRectFunc){
        ///Относительные ли UI координаты
        let that = this;
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        
        let num = this.graphics.push(this.game.add.graphics(0,0));
        if(isBorder){
            this.graphics[num -1].lineStyle(sizeBorder, fillBorder, alphaBorder);
        }
        if(isFill){
            this.graphics[num -1].beginFill(fillColor);
        }
        this.graphics[num -1].drawRect(coordX,coordY,w,h);
        if(isFill){
            this.graphics[num -1].endFill();
        }
        this.graphics[num -1].alpha = alphaFill;
        if(inputEnable){
            this.graphics[num -1].inputEnabled = true;
            this.graphics[num -1].input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
        this.graphics[num -1].fixedToCamera = this.fixedToCamera;
        this.layers[layer].add(this.graphics[num -1]);
        
        this.addUpdate(function(){
            ///Относительные ли UI координаты
            let newRect = newRectFunc();
            let coordX,coordY;
            if(relativCoord){
                coordX = that.x+(newRect.x);
                coordY = that.y+(newRect.y);
            }
            else{
                coordX = (x - (newRect.width));
                coordY = (y - (newRect.height));
            }
            that.graphics[num -1].clear();
            if(isBorder){
                that.graphics[num -1].lineStyle(sizeBorder, fillBorder, alphaBorder);
            }
            if(isFill){
                that.graphics[num -1].beginFill(fillColor);
            }
            that.graphics[num -1].drawRect(coordX,coordY,newRect.width,newRect.height);
            if(isFill){
                that.graphics[num -1].endFill();
            }
            that.graphics[num -1].alpha = alphaFill;
            if(inputEnable){
                that.graphics[num -1].inputEnabled = true;
                that.graphics[num -1].input.priorityID = that.inputPriority + (layer/that.layersCount);
            }
        });
        return this.graphics[num -1];
    }
    addUpdatedRoundRect(layer,relativCoord,x,y,w,h,isFill,isBorder,fillColor,fillBorder,sizeBorder,alphaFill,alphaBorder,inputEnable,radius,newRectFunc){
        
        let that = this;
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        
        let num = this.graphics.push(this.game.add.graphics(0,0));
        let grath = this.graphics[num -1];
        function addRect(x,y,width,height){
            
            if(isBorder){
                grath.lineStyle(sizeBorder, fillBorder, alphaBorder);
            }
            if(isFill){
                grath.beginFill(fillColor);

            }
            let rect = grath.drawRoundedRect(x,y,width,height,radius);
            if(isFill){
                grath.endFill();
            }
            if(inputEnable){
                grath.inputEnabled = true;

                grath.input.priorityID = that.inputPriority + (layer/that.layersCount);

            }

            
            
        }
        grath.alpha = alphaFill;
        this.layers[layer].add(grath);
        grath.fixedToCamera = this.fixedToCamera;
        let oldUiX = this.x;
        let oldUiY = this.y;
        let oldUiWidth = this.width;
        let oldUiHeight = this.height;
        let lastUIX = this.x;
        let lastUIY = this.y;
        this.addUpdate(function(){
            let newRect = newRectFunc();
            let coordX,coordY;
            if(relativCoord){
                coordX = that.x+(newRect.x);
                coordY = that.y+(newRect.y);
            }
            else{
                coordX = (x - (newRect.width));
                coordY = (y - (newRect.height));
            }
            grath.clear();
            if(( newRect.width > 0) && ( newRect.height > 0))
                addRect(coordX,coordY,newRect.width,newRect.height);
            
        });
        
        addRect(coordX,coordY,w,h);
        return grath;
    }
    /*
    *******Добавить Круг Элементу интерфейса********
    layer - номер слоя (int)
    relativCoord - координаты относительно текущего элемента (true/false)
    x,y - координаты(int)
    diameter - диаметр окружности (int)
    isFill - закрашен ли круг (true/false)
    isBorder - есть ли рамка(true/false)
    fillColor - цвет заливки (пример 0x000000)
    fillBorder - цвет рамки (пример 0x000000)
    sizeBorder -размер рамки (int)
    alphaFill - прозрачность заливки (от 0 до 1)
    alphaBorder - прозрачность рамки (от 0 до 1)
    inputEnable - возможность input (true,false)
    */
    addCircle(layer,relativCoord,x,y,diameter,isFill,isBorder,fillColor,fillBorder,alphaFill,sizeBorder,alphaBorder,inputEnable){
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        
        let num = this.graphics.push(this.game.add.graphics(0,0));
        if(isBorder){
            this.graphics[num -1].lineStyle(sizeBorder, fillBorder, alphaBorder);
        }
        if(isFill){
            this.graphics[num -1].beginFill(fillColor);
        }
        this.graphics[num -1].drawCircle(coordX,coordY,diameter);
        if(isFill){
            this.graphics[num -1].endFill();
        }
        this.graphics[num -1].alpha = alphaFill;
        if(inputEnable){
            this.graphics[num -1].inputEnabled = true;
            this.graphics[num -1].input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
        this.graphics[num -1].fixedToCamera = this.fixedToCamera;
        this.layers[layer].add(this.graphics[num -1]);
        return this.graphics[num -1];
    }
    /*
    *******Добавить Картинку Элементу интерфейса********
    layer - номер слоя (int)
    relativCoord - координаты относительно текущего элемента (true/false)
    x,y - координаты(int)
    w,h - размеры(int)
    imageKey - ключ картинки(string)
    alpha - прозрачность (от 0 до 1)
    inputEnable - возможность input (true,false)
    */
    addSprite(layer,relativCoord,x,y,w,h,imageKey,alpha,inputEnable){
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        
        let num = this.images.push(this.layers[layer].create(coordX,coordY,imageKey));
        this.images[num -1].anchor.setTo(0.5,0.5);
        this.images[num -1].height = h;
        this.images[num -1].width = w;
        this.images[num -1].alpha = alpha;
        if(inputEnable){
            this.images[num -1].inputEnabled = true;
            this.images[num -1].input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
        this.images[num -1].fixedToCamera = this.fixedToCamera;
        return this.images[num -1];
    }
    addImageWithMask(layer,relativCoord,x,y,w,h,imageKey,maskKey,alpha,inputEnable){
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        let maskW = w -1;
        let maskH = h -1;
        var bmd = game.make.bitmapData(maskW, maskH);

        //	And create an alpha mask image by combining pic and mask from the cache
        bmd.alphaMask(imageKey,maskKey,{width:maskW,height:maskH},{width:maskW,height:maskH});
        //	A BitmapData is just a texture. You need to apply it to a sprite or image
        //	to actually display it:
        let num = this.images.push(this.layers[layer].add(this.game.add.image(coordX,coordY,bmd)));
        this.images[num -1].anchor.setTo(0.5,0.5);
        this.images[num -1].height = maskH;
        this.images[num -1].width = maskW;
        this.images[num -1].alpha = alpha;
        if(inputEnable){
            this.images[num -1].inputEnabled = true;
            this.images[num -1].input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
        this.images[num -1].fixedToCamera = this.fixedToCamera;
        
        return this.images[num -1];
    }
    addImageWithMaskAndBorder(layer,relativCoord,x,y,w,h,imageKey,maskKey,borderKey,alpha,inputEnable){
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        let maskW = w -2;
        let maskH = h -2;
        var bmd = game.make.bitmapData(maskW, maskH);

        //	And create an alpha mask image by combining pic and mask from the cache
        bmd.alphaMask(imageKey,maskKey,{width:maskW,height:maskH},{width:maskW,height:maskH});
        //	A BitmapData is just a texture. You need to apply it to a sprite or image
        //	to actually display it:
        
        let num = this.images.push(this.layers[layer].add(this.game.add.image(coordX,coordY,bmd)));
        let imageSprite = this.images[num -1];
        imageSprite.anchor.setTo(0.5,0.5);
        imageSprite.height = maskH;
        imageSprite.width = maskW;
        imageSprite.alpha = alpha;
        if(inputEnable){
            imageSprite.inputEnabled = true;
            imageSprite.input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
        imageSprite.fixedToCamera = this.fixedToCamera;
        
        num = this.images.push(this.layers[layer].add(this.game.add.image(coordX,coordY,borderKey)));
        let borderSprite = this.images[num -1];
        borderSprite.anchor.setTo(0.5,0.5);
        borderSprite.height = h;
        borderSprite.width = w;
        borderSprite.alpha = alpha;
        if(inputEnable){
            borderSprite.inputEnabled = true;
            borderSprite.input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
        borderSprite.fixedToCamera = this.fixedToCamera;
        
        return {image:imageSprite,border:borderSprite};
    }
    addImage(layer,relativCoord,x,y,w,h,imageKey,alpha,inputEnable){
        let that =this;
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        
        let num = this.images.push(this.layers[layer].add(this.game.add.image(coordX,coordY,imageKey)));
        let image = this.images[num -1];
        this.images[num -1].anchor.setTo(0.5,0.5);
        this.images[num -1].height = h;
        this.images[num -1].width = w;
        this.images[num -1].alpha = alpha;
        if(inputEnable){
            this.images[num -1].inputEnabled = true;
            this.images[num -1].input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
        this.images[num -1].fixedToCamera = this.fixedToCamera;
        let lastUIX = this.x;
        let lastUIY = this.y;
            
        this.addUpdate(function(){
            if((that.x !== lastUIX) || (that.y !== lastUIY)){
                let diffX = that.x - lastUIX;
                let diffY = that.y - lastUIY;
                image.x += diffX;
                image.y += diffY;

                lastUIX = that.x;
                lastUIY = that.y;
                //diffX = that.x - that.parentUI.x;
                //diffY = that.y - that.parentUI.y;
            }
        })
        return this.images[num -1];
    }
    addImageWithDraggableBorder(layer,relativCoord,x,y,w,h,imageKey,alpha,borderColor){
        let that = this;
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        
        function forEachPixel(pixel) {		
          // processPixelRGB won't take an argument, so we've set our sillhouetteColor globally	
            pixel.r = 255;
            pixel.g = 255;
            pixel.b = 255;
            return pixel
        }
        function createSillhouette(srcKey) {
            var bmd = game.make.bitmapData()
            // load our texture into the bitmap
            bmd.load(srcKey)
            bmd.processPixelRGB(forEachPixel, this)
          return bmd
        }
        let sillhouetteBMD = createSillhouette(imageKey);
        ///Border
        let numBorder = this.images.push(this.layers[layer].add(this.game.add.image(coordX,coordY,sillhouetteBMD)));
        this.images[numBorder -1].anchor.setTo(0.5,0.5);
        this.images[numBorder -1].height = h + 5;
        this.images[numBorder -1].width = w + 5;
        //this.images[numBorder -1].scale.setTo(0.52,0.52);
        //0xFFD700;
        this.images[numBorder -1].tint=borderColor;
        this.images[numBorder -1].visible=false;
   
        
        //
        let num = this.images.push(this.layers[layer].add(this.game.add.image(coordX,coordY,imageKey)));
        this.images[num -1].anchor.setTo(0.5,0.5);
        this.images[num -1].height = h;
        this.images[num -1].width = w;
        this.images[num -1].alpha = alpha;
        this.images[num -1].inputEnabled = true;
        this.images[num -1].input.priorityID = this.inputPriority + (layer/this.layersCount);
        
        this.images[num -1].events.onDragStart.add(function(pointer){
               that.images[numBorder -1].visible = true;
        }, that);
        this.images[num -1].events.onDragUpdate.add(function(pointer){
            
        }, that);
        this.images[num -1].events.onDragStop.add(function(pointer){
               that.images[numBorder -1].visible = false;
        }, that);
    
        this.images[num -1].events.onInputOver.add(function(){
            that.images[numBorder -1].visible = true;
        }, that);
        
        this.images[num -1].events.onInputOut.add(function(){
            that.images[numBorder -1].visible = false;
        }, that);
        
        this.images[num -1].fixedToCamera = this.fixedToCamera;
        this.addUpdate(function(){
            that.images[numBorder -1].x = that.images[num -1].x; 
            that.images[numBorder -1].y = that.images[num -1].y; 
        });
        this.images[num -1].BORDER = this.images[numBorder -1];
        return this.images[num -1];
    }
    addImageWithBorder(layer,relativCoord,x,y,w,h,imageKey,alpha,borderColor){
        let that = this;
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        
        function forEachPixel(pixel) {		
          // processPixelRGB won't take an argument, so we've set our sillhouetteColor globally	
            pixel.r = 255;
            pixel.g = 255;
            pixel.b = 255;
            return pixel
        }
        function createSillhouette(srcKey) {
            var bmd = game.make.bitmapData()
            // load our texture into the bitmap
            bmd.load(srcKey)
            bmd.processPixelRGB(forEachPixel, this)
          return bmd
        }
        let sillhouetteBMD = createSillhouette(imageKey);
        ///Border
        let numBorder = this.images.push(this.layers[layer].add(this.game.add.image(coordX,coordY,sillhouetteBMD)));
        this.images[numBorder -1].anchor.setTo(0.5,0.5);
        this.images[numBorder -1].height = h + 5;
        this.images[numBorder -1].width = w + 5;
        //this.images[numBorder -1].scale.setTo(0.52,0.52);
        //0xFFD700;
        this.images[numBorder -1].tint=borderColor;
        this.images[numBorder -1].visible=false;
   
        
        //
        let num = this.images.push(this.layers[layer].add(this.game.add.image(coordX,coordY,imageKey)));
        this.images[num -1].anchor.setTo(0.5,0.5);
        this.images[num -1].height = h;
        this.images[num -1].width = w;
        this.images[num -1].alpha = alpha;
        this.images[num -1].inputEnabled = true;
        this.images[num -1].input.priorityID = this.inputPriority + (layer/this.layersCount);
        this.images[num -1].events.onInputOver.add(function(){
            that.images[numBorder -1].visible = true;
        }, that);
        
        this.images[num -1].events.onInputOut.add(function(){
            that.images[numBorder -1].visible = false;
        }, that);
        
        this.images[num -1].fixedToCamera = this.fixedToCamera;
        return this.images[num -1];
    }
    ///Добавить картинку с рамкой и которая при наведении увеличивает слой
    addImageWithBorderAndMove(layer,relativCoord,x,y,w,h,imageKey,alpha,borderColor){
        let that = this;
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        
        function forEachPixel(pixel) {		
          // processPixelRGB won't take an argument, so we've set our sillhouetteColor globally	
            pixel.r = 255;
            pixel.g = 255;
            pixel.b = 255;
            return pixel
        }
        function createSillhouette(srcKey) {
            var bmd = game.make.bitmapData()
            // load our texture into the bitmap
            bmd.load(srcKey)
            bmd.processPixelRGB(forEachPixel, this)
          return bmd
        }
        let sillhouetteBMD = createSillhouette(imageKey);
        ///Border
        let numBorder = this.images.push(this.layers[layer].add(this.game.add.image(coordX,coordY,sillhouetteBMD)));
        this.images[numBorder -1].anchor.setTo(0.5,0.5);
        this.images[numBorder -1].height = h;
        this.images[numBorder -1].width = w;
        this.images[numBorder -1].scale.setTo(0.52,0.52);
        //0xFFD700;
        this.images[numBorder -1].tint=borderColor;
        this.images[numBorder -1].visible=false;
        //
        let num = this.images.push(this.layers[layer+1].add(this.game.add.image(coordX,coordY,imageKey)));
        this.images[num -1].anchor.setTo(0.5,0.5);
        this.images[num -1].height = h;
        this.images[num -1].width = w;
        this.images[num -1].alpha = alpha;
        this.images[num -1].inputEnabled = true;
        this.images[num -1].input.priorityID = this.inputPriority + (layer/this.layersCount);
        console.log(this.images[num -1].width)
        this.images[num -1].events.onInputOver.add(function(){
            that.images[numBorder -1].visible = true;
            that.images[num -1].moveUp();
            that.images[numBorder -1].moveUp();
            console.log('top')
        }, that);
        
        this.images[num -1].events.onInputOut.add(function(){
            that.images[numBorder -1].visible = false;
            that.images[num -1].moveDown();
            that.images[numBorder -1].moveDown();
            console.log('down')
        }, that);
        this.images[num -1].fixedToCamera = this.fixedToCamera;
        return this.images[num -1];
    }
    /*
    *******Добавить Слайдэр Элементу интерфейса********
    layer - номер слоя (int)
    x1 - начальная координата x слайдера(int)
    x2 - конечная координата x слайдера(int)
    y - координата(int)
    min - минимальное значение слайдера (int)
    max - максимальное значение слайдера (int)
    fast - ускоренная формула (true,false)
    */
    addGorizontalSlider(layer,x1,x2,y,min,max,fast){
        let that = this;
        x1 += this.x;
        x2 += this.x;
        y += this.y;
        var line = this.game.add.graphics(0,0);
        line.lineStyle(2, 0x4100E0, 0.7);
        line.moveTo(x1, y);
        line.lineTo(x2, y);
        line.fixedToCamera = this.fixedToCamera;
        this.layers[layer].add(line);
        let current = Math.round(((max + min)/2));
        let startX = (x1 + x2)/2;
        if(fast)
            startX = x1;
        var sprite = this.layers[layer].create((startX),(y),'slider');
        sprite.anchor.setTo(0.5,0.5);
        sprite.height = 25;
        sprite.width = 25;
        sprite.fixedToCamera = this.fixedToCamera;
        
        let num = this.sliders.push({scale:{line:line,x1:x1,x2:x2,y:y},slider:sprite,value:{max:max,min:min,current:current}});
        
        
        sprite.inputEnabled = true;
        sprite.input.priorityID = this.inputPriority + (layer/this.layersCount);
        sprite.input.useHandCursor = true;
        sprite.input.enableDrag();
        sprite.input.allowVerticalDrag = false;
        sprite.events.onDragUpdate.add(function(){
            let slider = that.sliders[num - 1]
            if(sprite.x <= x1){
                sprite.x = x1;
                 
            }    
            if(sprite.x >= x2){
                sprite.x = x2;
            }
            if(!fast){
                slider.value.current = ((sprite.x - slider.scale.x1)* ((slider.value.max - slider.value.min)/(slider.scale.x2 - slider.scale.x1)))+ slider.value.min;
            }
            else{
                //У=(Х-Хмин))*(Х-Хмин))*(Ymax-Ymin)/(Xmax-Xmin)/(Xmax-Xmin) +Умин
               slider.value.current = (sprite.x - slider.scale.x1)*(sprite.x - slider.scale.x1)*(slider.value.max - slider.value.min)/(slider.scale.x2 - slider.scale.x1)+ slider.value.min; 
            }
            
            slider.value.current = Math.round(slider.value.current);
            //y = x * (Ymax-Ymin)/(Xmax-Xmin)/(Xmax-Xmin)
           
        }, this);
        
        //this.sliders[num -1].fixedToCamera = this.isFixed;
        return this.sliders[num -1];
        
        
        
    }
    //Вертикальный слайдер 
    addVerticalSlider(layer,relativCoord,y1,y2,x,w,h,borderSize,borderFill,borderAlpha,sliderKey){
        let that = this;
        if(relativCoord){
            y1 += this.y;
            y2 += this.y;
            x += this.x;
        }
        let numG = this.graphics.push(this.game.add.graphics(0,0));
        let line = this.graphics[numG -1];
        line.lineStyle(borderSize, borderFill, borderAlpha);
        line.moveTo(x, y1);
        line.lineTo(x, y2);
        line.fixedToCamera = this.fixedToCamera;
        y1 = y1 + h/2;
        y2 = y2 - h/2;
        this.layers[layer].add(line);
        let startY = (y1 + y2)/2;
        var sprite = this.addImage(layer,false,x,startY,w,h,sliderKey,1,true);
        //var sprite = this.layers[layer].create((x),(startY),sliderKey);
        //sprite.anchor.setTo(0.5,0.5);
        //sprite.height = h;
        //sprite.width = w;
        let num = this.sliders.push({percent:0});
        //sprite.inputEnabled = true;
        sprite.input.priorityID = this.inputPriority + (layer/this.layersCount);
        sprite.input.useHandCursor = true;
        sprite.input.enableDrag();
        sprite.input.allowHorizontalDrag = false;
        function updateSprite (){
            if(sprite.y <= y1 ){
                sprite.y = y1;
                that.sliders[num -1].percent = 0;
            }    
            if(sprite.y >= y2){
                sprite.y = y2;
                that.sliders[num -1].percent = 100;
            }
        }
        sprite.events.onInputDown.add(function(){
            sprite.frame = 2;
        }, that);
         sprite.events.onInputUp.add(function(){
             sprite.frame = 1;
        }, that);
         sprite.events.onInputOver.add(function(){
             sprite.frame = 1;
        }, that);
         sprite.events.onInputOut.add(function(){
             sprite.frame = 0;
        }, that);
        sprite.events.onDragUpdate.add(function(){

            updateSprite ();
            this.sliders[num -1].percent =  (sprite.y - y1)/((y2 - y1)/100) ;
        }, this);
        this.onUpdate.push(function(){
        
            sprite.y = y1 + that.sliders[num -1].percent*((y2 - y1)/100);
            updateSprite ();
        });
        
        //this.sliders[num -1].fixedToCamera = this.isFixed;
        return this.sliders[num -1];
        
        
        
    }
    
    ///Добавить Виджет с текстом и картинками всместо текста
    addUIWithBitmapTextAndImage(layer,relativCoord,x,y,width,align,string,wordTypes,style,stringSize,alpha,inputEnable){
        ///Относительные ли UI координаты
        
        let that = this;
        let mainWidget = null;
        ///Получаем массив из строки. Делим по пробелам
        let textArray = string.split(' ');
        let stringWidth = 0;
        ///Массив строк текстоов
        let uiArray = [];
        ////Создаем основной виджет
        mainWidget = this.addUI(layer,relativCoord,x,y,width,0,10)
        //mainWidget = that.GUI.addUI(x,y,width,0,null,layer,10);
        //mainWidget.addBackgroundRoundedRect(0,true,true,0x066000,0xFFFFFF,1,1,1,15,false);
        ////Создаем линию с виждетом и кидаем
        let num = uiArray.push(mainWidget.addUI(1,true,stringSize,stringSize,mainWidget.width*0.9,stringSize,10));
        let currentUi = uiArray[num -1]; 
        //secondWidget.widget.addBackgroundRoundedRect(0,true,true,0xCCF00A,0xFFFFFF,1,1,1,15,false);
        let lastWidgetWidth = 0;
        ///Перебераем массив слов
        for(let i in textArray){
            ///Если есть слово </n> - это перевод строки
            let pos = textArray[i].indexOf('</n>');
            let newLine = false;
            if(pos !== -1){
                newLine = true;
            }
            ///Получаем массив слов из каждого слова (в нем могут быть запятые и подобное) из строки.
            let words = textArray[i].split(/[ .:;?!~,'"&|()<>{}\[\]\r\n/\\]+/);
            let isInWording = false;
            console.log(style)
            let wordStyle = style;
            let isImage = false;
            let imageKey = false;
            let frame = 0;
            let object;
            console.log(style)
            ///Определеяем соотношение полученых слов с теми что были переданы в функию
            for(let n in words){
                ///Проверяем совпадает ли это слово с wordTypes
                for(let j in wordTypes){
                    ///Проверяем это слово или нет
                    if(words[n] == wordTypes[j].word){
                         console.log(wordTypes)
                        console.log(wordTypes[j])
                        console.log(words[n])
                        isInWording = true;
                        if(wordTypes[j].isWord){
                            
                            wordStyle = wordTypes[j].style;
                        } 
                        else{
                            isImage = true;
                            imageKey = wordTypes[j].imageKey;
                            frame = wordTypes[j].frame;
                        }

                    }
                }
            }
            console.log(wordStyle)
            ///Если "Длина строки" слишком длинная или это новая строка
            if((stringWidth >= mainWidget.width - stringSize*2) || (newLine)){
                
                currentUi.width = stringWidth;
                num = uiArray.push(mainWidget.addUI(1,true,stringSize,stringSize+stringSize*(uiArray.length),mainWidget.width*0.9,stringSize,10));
                currentUi = uiArray[num -1];
                //currentUi.widget.addBackgroundRoundedRect(0,true,true,0xCCF00A,0xFFFFFF,1,1,1,15,false);
                stringWidth = 0;
            }

            if(!newLine){
                if(isImage){
                    ////Если это картинка добавляем ее
                    object = currentUi.addImage(2,true,stringWidth+(stringSize/4),stringSize/2,stringSize,stringSize,imageKey,alpha,false);
                    object.frame = frame;
                    object.anchor.setTo(0.5,0.5);
                }else{
                    ////Если это текст добавляем его
                    
                    object = currentUi.addBitMapText(2,true,stringWidth+(stringSize/4),stringSize/2,textArray[i],wordStyle.bitmapFontKey,wordStyle.fontSize,alpha,false)
                    object.tint = this.hexToOct(wordStyle.fill);
                    object.anchor.setTo(0.5,0.5);
                }
                object.x = object.x + object.width/2;
                stringWidth += (object.width + stringSize/4);
            }

        }
        currentUi.width = stringWidth;
        ///Сортируем и упорядочаем 
        let allWidgetsHeight = 0;
        let maxWidgetWidth = 0;
        let padding = 30;
        ///Получаем высоту и ширину главного виджета
        for(let i in uiArray){
            allWidgetsHeight += stringSize;
            if(uiArray[i].width > maxWidgetWidth){
                maxWidgetWidth = uiArray[i].width;
            }
            uiArray[i].height = stringSize;
        }
        mainWidget.height = allWidgetsHeight + padding;    
        mainWidget.width = maxWidgetWidth + padding; 
        ////выравниваем дочерние виджеты под Главный
        if(align == 'center'){
            for(let i in uiArray){
                uiArray[i].x = mainWidget.x + (mainWidget.width/2) - (uiArray[i].width/2)
                uiArray[i].y = padding/2 + mainWidget.y + (i*uiArray[i].height);
            }
            mainWidget.x = mainWidget.x - (mainWidget.width/2);
            mainWidget.y = mainWidget.y - (mainWidget.height/2);
        }else if(align == 'left'){
            for(let i in uiArray){
                uiArray[i].x = mainWidget.x + (mainWidget.width/2) - maxWidgetWidth/2;
                uiArray[i].y = padding/2 + mainWidget.y + (i*uiArray[i].height);
            }
            mainWidget.y = mainWidget.y - (mainWidget.height/2);
        }
        
        return mainWidget;
    }
    ///Добавить Виджет с текстом и картинками всместо текста
    addUIWithTextAndImage(layer,relativCoord,x,y,width,align,string,wordTypes,style,stringSize,alpha,inputEnable){
        ///Относительные ли UI координаты
        
        let that = this;
        let mainWidget = null;
        ///Получаем массив из строки. Делим по пробелам
        let textArray = string.split(' ');
        let stringWidth = 0;
        ///Массив строк текстоов
        let uiArray = [];
        ////Создаем основной виджет
        mainWidget = this.addUI(layer,relativCoord,x,y,width,0,10)
        //mainWidget = that.GUI.addUI(x,y,width,0,null,layer,10);
        //mainWidget.addBackgroundRoundedRect(0,true,true,0x066000,0xFFFFFF,1,1,1,15,false);
        ////Создаем линию с виждетом и кидаем
        let num = uiArray.push(mainWidget.addUI(1,true,stringSize,stringSize,mainWidget.width*0.9,stringSize,10));
        let currentUi = uiArray[num -1]; 
        //secondWidget.widget.addBackgroundRoundedRect(0,true,true,0xCCF00A,0xFFFFFF,1,1,1,15,false);
        let lastWidgetWidth = 0;
        ///Перебераем массив слов
        for(let i in textArray){
            ///Если есть слово </n> - это перевод строки
            let pos = textArray[i].indexOf('</n>');
            let newLine = false;
            if(pos !== -1){
                newLine = true;
            }
            ///Получаем массив слов из каждого слова (в нем могут быть запятые и подобное) из строки.
            let words = textArray[i].split(/[ .:;?!~,'"&|()<>{}\[\]\r\n/\\]+/);
            let isInWording = false;
            let wordStyle = style;
            let isImage = false;
            let imageKey = false;
            let frame = 0;
            let object;
            ///Определеяем соотношение полученых слов с теми что были переданы в функию
            for(let n in words){
                ///Проверяем совпадает ли это слово с wordTypes
                for(let j in wordTypes){
                    ///Проверяем это слово или нет
                    if(words[n] == wordTypes[j].word){
                        isInWording = true;
                        if(wordTypes[j].isWord){
                            wordStyle = wordTypes[j].style;
                        } 
                        else{
                            isImage = true;
                            imageKey = wordTypes[j].imageKey;
                            frame = wordTypes[j].frame;
                        }

                    }
                }
            }
            
            ///Если "Длина строки" слишком длинная или это новая строка
            if((stringWidth >= mainWidget.width - stringSize*2) || (newLine)){
                
                currentUi.width = stringWidth;
                num = uiArray.push(mainWidget.addUI(1,true,stringSize,stringSize+stringSize*(uiArray.length),mainWidget.width*0.9,stringSize,10));
                currentUi = uiArray[num -1];
                //currentUi.widget.addBackgroundRoundedRect(0,true,true,0xCCF00A,0xFFFFFF,1,1,1,15,false);
                stringWidth = 0;
            }

            if(!newLine){
                if(isImage){
                    ////Если это картинка добавляем ее
                    object = currentUi.addImage(2,true,stringWidth+(stringSize/4),stringSize/2,stringSize,stringSize,imageKey,alpha,false);
                    object.frame = frame;
                    object.anchor.setTo(0.5,0.5);
                }else{
                    ////Если это текст добавляем его
                    object = currentUi.addText(2,true,stringWidth+(stringSize/4),stringSize/2,textArray[i],wordStyle,alpha,false);
                    object.anchor.setTo(0.5,0.5);
                }
                object.x = object.x + object.width/2;
                stringWidth += (object.width + stringSize/4);
            }

        }
        currentUi.width = stringWidth;
        ///Сортируем и упорядочаем 
        let allWidgetsHeight = 0;
        let maxWidgetWidth = 0;
        ///Получаем высоту и ширину главного виджета
        for(let i in uiArray){
            allWidgetsHeight += stringSize;
            if(uiArray[i].width > maxWidgetWidth){
                maxWidgetWidth = uiArray[i].width;
            }
            uiArray[i].height = stringSize;
        }
        mainWidget.height = allWidgetsHeight + stringSize*2;    
        mainWidget.width = maxWidgetWidth + stringSize*2; 
        ////выравниваем дочерние виджеты под Главный
        if(align == 'center'){
            for(let i in uiArray){
                uiArray[i].x = mainWidget.x + (mainWidget.width/2) - (uiArray[i].width/2)
            }
        }else if(align == 'left'){
            for(let i in uiArray){
                uiArray[i].x = mainWidget.x + (mainWidget.width/2) - maxWidgetWidth/2;
            }
        }
        mainWidget.x = mainWidget.x - (mainWidget.width/2);
        mainWidget.y = mainWidget.y - (mainWidget.height/2);
        return mainWidget;
    }
    ///Добавить Виджет с текстом и картинками всместо текста
    addWidgetWithBitmapTextAndImage(layer,x,y,width,align,string,wordTypes,style,stringSize,alpha,inputEnable){
        ///Относительные ли UI координаты
        
        let that = this;
        let mainWidget = null;
        ///Получаем массив из строки. Делим по пробелам
        let textArray = string.split(' ');
        let stringWidth = 0;
        ///Массив строк текстоов
        let uiArray = [];
        ////Создаем основной виджет
        mainWidget = that.GUI.addUI(x,y,width,0,null,layer,10);
        //mainWidget.addBackgroundRoundedRect(0,true,true,0x066000,0xFFFFFF,1,1,1,15,false);
        ////Создаем линию с виждетом и кидаем
        let num = uiArray.push(mainWidget.addUI(1,true,stringSize,stringSize,mainWidget.width*0.9,stringSize,10));
        let currentUi = uiArray[num -1]; 
        //secondWidget.widget.addBackgroundRoundedRect(0,true,true,0xCCF00A,0xFFFFFF,1,1,1,15,false);
        let lastWidgetWidth = 0;
        ///Перебераем массив слов
        for(let i in textArray){
            ///Если есть слово </n> - это перевод строки
            let pos = textArray[i].indexOf('</n>');
            let newLine = false;
            if(pos !== -1){
                newLine = true;
            }
            ///Получаем массив слов из каждого слова (в нем могут быть запятые и подобное) из строки.
            let words = textArray[i].split(/[ .:;?!~,'"&|()<>{}\[\]\r\n/\\]+/);
            let isInWording = false;
            let wordStyle = style;
            let isImage = false;
            let imageKey = false;
            let frame = 0;
            let object;
            ///Определеяем соотношение полученых слов с теми что были переданы в функию
            for(let n in words){
                ///Проверяем совпадает ли это слово с wordTypes
                for(let j in wordTypes){
                    ///Проверяем это слово или нет
                    if(words[n] == wordTypes[j].word){
                        isInWording = true;
                        if(wordTypes[j].isWord){
                            wordStyle = wordTypes[j].style;
                        } 
                        else{
                            isImage = true;
                            imageKey = wordTypes[j].imageKey;
                            frame = wordTypes[j].frame;
                        }

                    }
                }
            }
            
            ///Если "Длина строки" слишком длинная или это новая строка
            if((stringWidth >= mainWidget.width - stringSize*2) || (newLine)){
                
                currentUi.width = stringWidth;
                num = uiArray.push(mainWidget.addUI(1,true,stringSize,stringSize+stringSize*(uiArray.length),mainWidget.width*0.9,stringSize,10));
                currentUi = uiArray[num -1];
                //currentUi.widget.addBackgroundRoundedRect(0,true,true,0xCCF00A,0xFFFFFF,1,1,1,15,false);
                stringWidth = 0;
            }

            if(!newLine){
                if(isImage){
                    ////Если это картинка добавляем ее
                    object = currentUi.addImage(2,true,stringWidth+(stringSize/4),stringSize/2,stringSize,stringSize,imageKey,alpha,false);
                    object.frame = frame;
                    object.anchor.setTo(0.5,0.5);
                }else{
                    ////Если это текст добавляем его
                    object = currentUi.addBitMapText(2,true,stringWidth+(stringSize/4),stringSize/2,textArray[i],wordStyle.bitmapFontKey,wordStyle.fontSize,alpha,false)
                    object.tint = this.hexToOct(wordStyle.fill);
                    //object = currentUi.addText(2,true,stringWidth+(stringSize/4),stringSize/2,textArray[i],wordStyle,alpha,false);
                    object.anchor.setTo(0.5,0.5);
                }
                object.x = object.x + object.width/2;
                stringWidth += (object.width + stringSize/4);
            }

        }
        currentUi.width = stringWidth;
        ///Сортируем и упорядочаем 
        let allWidgetsHeight = 0;
        let maxWidgetWidth = 0;
        let padding = 30;
        ///Получаем высоту и ширину главного виджета
        for(let i in uiArray){
            allWidgetsHeight += stringSize;
            if(uiArray[i].width > maxWidgetWidth){
                maxWidgetWidth = uiArray[i].width;
            }
            uiArray[i].height = stringSize;
        }
        mainWidget.height = allWidgetsHeight + padding;    
        mainWidget.width = maxWidgetWidth + padding; 
        ////выравниваем дочерние виджеты под Главный
        if(align == 'center'){
            for(let i in uiArray){
                uiArray[i].x = mainWidget.x + (mainWidget.width/2) - (uiArray[i].width/2)
                uiArray[i].y = padding/2 + mainWidget.y + (i*uiArray[i].height);
            }
        }else if(align == 'left'){
            for(let i in uiArray){
                uiArray[i].x = mainWidget.x + (mainWidget.width/2) - maxWidgetWidth/2;
                uiArray[i].y = padding/2 + mainWidget.y + (i*uiArray[i].height);
            }
        }
        return mainWidget;
    }
   ///Добавить Виджет с текстом и картинками всместо текста
    addWidgetWithTextAndImage(layer,x,y,width,align,string,wordTypes,style,stringSize,alpha,inputEnable){
        ///Относительные ли UI координаты
        
        let that = this;
        let mainWidget = null;
        ///Получаем массив из строки. Делим по пробелам
        let textArray = string.split(' ');
        let stringWidth = 0;
        ///Массив строк текстоов
        let uiArray = [];
        ////Создаем основной виджет
        mainWidget = that.GUI.addUI(x,y,width,0,null,layer,10);
        //mainWidget.addBackgroundRoundedRect(0,true,true,0x066000,0xFFFFFF,1,1,1,15,false);
        ////Создаем линию с виждетом и кидаем
        let num = uiArray.push(mainWidget.addUI(1,true,stringSize,stringSize,mainWidget.width*0.9,stringSize,10));
        let currentUi = uiArray[num -1]; 
        //secondWidget.widget.addBackgroundRoundedRect(0,true,true,0xCCF00A,0xFFFFFF,1,1,1,15,false);
        let lastWidgetWidth = 0;
        ///Перебераем массив слов
        for(let i in textArray){
            ///Если есть слово </n> - это перевод строки
            let pos = textArray[i].indexOf('</n>');
            let newLine = false;
            if(pos !== -1){
                newLine = true;
            }
            ///Получаем массив слов из каждого слова (в нем могут быть запятые и подобное) из строки.
            let words = textArray[i].split(/[ .:;?!~,'"&|()<>{}\[\]\r\n/\\]+/);
            let isInWording = false;
            let wordStyle = style;
            let isImage = false;
            let imageKey = false;
            let frame = 0;
            let object;
            ///Определеяем соотношение полученых слов с теми что были переданы в функию
            for(let n in words){
                ///Проверяем совпадает ли это слово с wordTypes
                for(let j in wordTypes){
                    ///Проверяем это слово или нет
                    if(words[n] == wordTypes[j].word){
                        isInWording = true;
                        if(wordTypes[j].isWord){
                            wordStyle = wordTypes[j].style;
                        } 
                        else{
                            isImage = true;
                            imageKey = wordTypes[j].imageKey;
                            frame = wordTypes[j].frame;
                        }

                    }
                }
            }
            
            ///Если "Длина строки" слишком длинная или это новая строка
            if((stringWidth >= mainWidget.width - stringSize*2) || (newLine)){
                
                currentUi.width = stringWidth;
                num = uiArray.push(mainWidget.addUI(1,true,stringSize,stringSize+stringSize*(uiArray.length),mainWidget.width*0.9,stringSize,10));
                currentUi = uiArray[num -1];
                //currentUi.widget.addBackgroundRoundedRect(0,true,true,0xCCF00A,0xFFFFFF,1,1,1,15,false);
                stringWidth = 0;
            }

            if(!newLine){
                if(isImage){
                    ////Если это картинка добавляем ее
                    object = currentUi.addImage(2,true,stringWidth+(stringSize/4),stringSize/2,stringSize,stringSize,imageKey,alpha,false);
                    object.frame = frame;
                    object.anchor.setTo(0.5,0.5);
                }else{
                    ////Если это текст добавляем его
                    object = currentUi.addText(2,true,stringWidth+(stringSize/4),stringSize/2,textArray[i],wordStyle,alpha,false);
                    object.anchor.setTo(0.5,0.5);
                }
                object.x = object.x + object.width/2;
                stringWidth += (object.width + stringSize/4);
            }

        }
        currentUi.width = stringWidth;
        ///Сортируем и упорядочаем 
        let allWidgetsHeight = 0;
        let maxWidgetWidth = 0;
        ///Получаем высоту и ширину главного виджета
        for(let i in uiArray){
            allWidgetsHeight += stringSize;
            if(uiArray[i].width > maxWidgetWidth){
                maxWidgetWidth = uiArray[i].width;
            }
            uiArray[i].height = stringSize;
        }
        mainWidget.height = allWidgetsHeight + stringSize*2;    
        mainWidget.width = maxWidgetWidth + stringSize*2; 
        ////выравниваем дочерние виджеты под Главный
        if(align == 'center'){
            for(let i in uiArray){
                uiArray[i].x = mainWidget.x + (mainWidget.width/2) - (uiArray[i].width/2)
            }
        }else if(align == 'left'){
            for(let i in uiArray){
                uiArray[i].x = mainWidget.x + (mainWidget.width/2) - maxWidgetWidth/2;
            }
        }
        
        return mainWidget;
    }
    ///Добавить всплывающую подсказку с текстом и картинками
    addTitleTextWithImages(width,rightleft,topbot,align,wordTypes,style,overObject,getString){
        ///Относительные ли UI координаты
        let that = this;
        
        let responseTime = 500;
        let closeTime = 20000;
        let isOver = false;
        let isOut = true;
        let startOverTime = null;
        ////0 Создаем массив для текстов
        /////1 Разбиваем на части строку - получаем массив
        /////2 Проверяем каждое слово с искомым массивом слов, создаем текст каждого слова в нужном стиле
        let mainWidget = null;
        function createTitle(string){
            if(mainWidget == null){
              
              mainWidget = that.addWidgetWithBitmapTextAndImage(null,that.game.input.mousePointer.x,that.game.input.mousePointer.y,width,align,string,wordTypes,style,32,0,false);
                //mainWidget = that.addWidgetWithTextAndImage(null,that.game.input.mousePointer.x,that.game.input.mousePointer.y,width,align,string,wordTypes,style,25,0,false);
               // mainWidget.addBackground(1,'test',1,false);
                let uiAlign = getAlign();
                move(uiAlign.rl,uiAlign.tb);
                //mainWidget.addBackgroundRect(0,true,true,0x000000,0xFFD700,1,0,1,false)
                mainWidget.addBackgroundRoundedRect(0,true,true,0x000000,0xFFD700,1,0,0.5,2,false);
                mainWidget.addTween(['texts','images'],{ alpha: 1 },500);
                //mainWidget.addTween(['graphics'],{ alpha: 0.8 },500);
                mainWidget.addTween(['graphics'],{ alpha: 0.9 },500);
                function getAlign(){
                    let rl = 'right';
                    let tb = 'top';
                    if( (that.game.input.mousePointer.x + mainWidget.width) > that.game.width){
                        rl = 'left';
                    }
                    ///Сдесь нужно отталкиваться от верхней точки либо иры либо камеры
                    if((that.game.input.mousePointer.y - mainWidget.height) < 0){
                        tb = 'bot';
                    }
                    return {rl:rl,tb:tb};
                }
                function move(rl,tb){
                    if(rl == 'right'){
                        if(tb == 'top'){
                            mainWidget.move(that.game.input.mousePointer.x,that.game.input.mousePointer.y - mainWidget.height);

                        }else if(tb == 'bot'){
                            mainWidget.move(that.game.input.mousePointer.x,that.game.input.mousePointer.y);

                        }

                    }else if(rl == 'left'){

                        if(tb == 'top'){
                            mainWidget.move(that.game.input.mousePointer.x - mainWidget.width,that.game.input.mousePointer.y - mainWidget.height);

                        }else if(tb == 'bot'){
                            mainWidget.move(that.game.input.mousePointer.x - mainWidget.width,that.game.input.mousePointer.y);

                        }
                    }
                };
                let lastCursorX = that.game.input.mousePointer.x;
                let lastCursorY = that.game.input.mousePointer.y;
                mainWidget.addUpdate(function(){
                    //Если положение курсора изменилось - двигаем тайтл
                    if((lastCursorX !== that.game.input.mousePointer.x) || (lastCursorY != that.game.input.mousePointer.y)){
                        lastCursorX = that.game.input.mousePointer.x;
                        lastCursorY = that.game.input.mousePointer.y;
                        uiAlign = getAlign();
                        move(uiAlign.rl,uiAlign.tb);
                    }
                })
                
            }
        }
        function deleteTitle(){
            if(mainWidget !== null){
                
                mainWidget.delete();
                mainWidget = null;
            }
        }
        function ifLeftTime(time){
            if( (new Date() - startOverTime) > time)
                return true;
            else return false
        }
        
        if(overObject.input.enabled == true){
            overObject.events.onInputOver.add(function(){
                isOver = true;
                startOverTime = new Date();
                isOut = false;
            },this);
            overObject.events.onInputOut.add(function(){
                isOver = false;
            },this);
        }
        that.onUpdate.push(function(){
           if(overObject.input.isDragged){
                    deleteTitle();
            }
            if(isOver){
                ///Если навели и время ответа прошло
                if( ifLeftTime(responseTime) ){
                    createTitle(getString())
                }
                //Если прошло слишком много времени удаляем ui
                if( ifLeftTime(closeTime) ){
                    deleteTitle();
                    isOver = false;
                }
            }else if(!isOut){
                deleteTitle();
                isOut = true;
            }
            
        });
        return mainWidget;
    }
    
    addTitleForArtifacts(width,rightleft,topbot,align,wordTypes,style,overObject,getString,getName){
        ///Относительные ли UI координаты
        let that = this;
        
        let responseTime = 500;
        let closeTime = 20000;
        let isOver = false;
        let isOut = true;
        let startOverTime = null;
        ////0 Создаем массив для текстов
        /////1 Разбиваем на части строку - получаем массив
        /////2 Проверяем каждое слово с искомым массивом слов, создаем текст каждого слова в нужном стиле
        let firstUi = null;
        function createTitle(string){
            console.log(firstUi)
            if(firstUi == null){
              
                
       
                function onClose(){
     
                    firstUi.delete();
                }
                let height = 200;
                let width = 400;
                let coords = getCoordinats(height,width);
                console.log(coords)
                firstUi = that.GUI.addUI(coords.x,coords.y,width,height,null,null,10);
                //firstUi.addBackgroundRect(0,true,true,0x000000,0xFFD700,1,1,1,false);
                //firstUi.addBackground(1,'ui_400_200_2',1,true);
                firstUi.addImage(1,true,firstUi.width*0.5,firstUi.height*0.5,600,400,'ui_400_200_2',0,false);
                
                /*
                firstUi.addFullWindowRect(0,0,0,that.game.width,that.game.height,true,0x000000,0.5,function(){
                    onClose();
                });
                */
                
                let ui = firstUi.addUI(1,true,firstUi.width/2 - 200,firstUi.height/2 - 100,400,200,15);
                
                let uiName = ui.addBitMapText(1,true,ui.width*0.5,+30,'Название','font_1',25,0,false);
                uiName.tint = 0xFFDAB9;
             
                ui.addUIWithBitmapTextAndImage(1,true,ui.width*0.5,ui.height*0.65,ui.width*0.95,'center',string,wordTypes,style,50,0,false);
                
               // that.addWidgetWithBitmapTextAndImage(null,that.game.input.mousePointer.x,that.game.input.mousePointer.y,width,align,string,wordTypes,style,50,0,false);
                //firstUi = that.addWidgetWithBitmapTextAndImage(null,that.game.input.mousePointer.x,that.game.input.mousePointer.y,width,align,string,wordTypes,style,50,0,false);
                //firstUi = that.addWidgetWithTextAndImage(null,that.game.input.mousePointer.x,that.game.input.mousePointer.y,width,align,string,wordTypes,style,25,0,false);
               // firstUi.addBackground(1,'test',1,false);
                //let uiAlign = getAlign();
                //move(uiAlign.rl,uiAlign.tb);
                //firstUi.addBackgroundRect(0,true,true,0x000000,0xFFD700,1,0,1,false)
                //firstUi.addBackgroundRoundedRect(0,true,true,0x000000,0xFFD700,1,0,0.5,2,false);
                firstUi.addTween(['texts','images','graphics'],{ alpha: 1 },500);
                //firstUi.addTween(['graphics'],{ alpha: 0.8 },500);
                //firstUi.addTween(['graphics'],{ alpha: 0.9 },500);
                
                
                function getAlign(){
                    let rl = 'right';
                    let tb = 'top';
                    if( (that.game.input.mousePointer.x + 600) > that.game.width){
                        rl = 'left';
                    }
                    ///Сдесь нужно отталкиваться от верхней точки либо иры либо камеры
                    if((that.game.input.mousePointer.y - 400) < 0){
                        tb = 'bot';
                    }
                    return {rl:rl,tb:tb};
                }
                function move(rl,tb){
                    if(rl == 'right'){
                        if(tb == 'top'){
                            firstUi.move(that.game.input.mousePointer.x,that.game.input.mousePointer.y - firstUi.height);

                        }else if(tb == 'bot'){
                            firstUi.move(that.game.input.mousePointer.x,that.game.input.mousePointer.y);

                        }

                    }else if(rl == 'left'){

                        if(tb == 'top'){
                            firstUi.move(that.game.input.mousePointer.x - firstUi.width,that.game.input.mousePointer.y - firstUi.height);

                        }else if(tb == 'bot'){
                            firstUi.move(that.game.input.mousePointer.x - firstUi.width,that.game.input.mousePointer.y);

                        }
                    }
                };
                function getCoordinats(h,w){
                    let align = getAlign();
                    console.log(align)
                    let rl = align.rl;
                    let tb = align.tb;
                    let x,y;
                    if(rl == 'right'){
                        if(tb == 'top'){
                            x = that.game.input.mousePointer.x;
                            y = that.game.input.mousePointer.y - h;

                        }else if(tb == 'bot'){
                            x = that.game.input.mousePointer.x;
                            y = that.game.input.mousePointer.y;
                        }

                    }else if(rl == 'left'){

                        if(tb == 'top'){
                            x = that.game.input.mousePointer.x - w;
                            y = that.game.input.mousePointer.y - h;

                        }else if(tb == 'bot'){
                            x = that.game.input.mousePointer.x - w;
                            y = that.game.input.mousePointer.y;

                        }
                    }
                    return {x:x,y:y};
                }
                let lastCursorX = that.game.input.mousePointer.x;
                let lastCursorY = that.game.input.mousePointer.y;
                firstUi.addUpdate(function(){
                    //Если положение курсора изменилось - двигаем тайтл
                    
                    if((lastCursorX !== that.game.input.mousePointer.x) || (lastCursorY != that.game.input.mousePointer.y)){
                        lastCursorX = that.game.input.mousePointer.x;
                        lastCursorY = that.game.input.mousePointer.y;
                        //uiAlign = getAlign();
                        //move(uiAlign.rl,uiAlign.tb);
                        deleteTitle();
                        createTitle(string);
                    }
                    
                })
                
            }
        }
        function deleteTitle(){
            if(firstUi !== null){
                
                firstUi.delete();
                firstUi = null;
            }
        }
        function ifLeftTime(time){
            if( (new Date() - startOverTime) > time)
                return true;
            else return false
        }
        
        if(overObject.input.enabled == true){
            overObject.events.onInputOver.add(function(){
                isOver = true;
                startOverTime = new Date();
                isOut = false;
            },this);
            overObject.events.onInputOut.add(function(){
                isOver = false;
            },this);
        }
        that.onUpdate.push(function(){
           if(overObject.input.isDragged){
                    deleteTitle();
            }
            if(isOver){
                ///Если навели и время ответа прошло
                if( ifLeftTime(responseTime) ){
                    console.log('TEST')
                    createTitle(getString())
                }
                //Если прошло слишком много времени удаляем ui
                if( ifLeftTime(closeTime) ){
                    deleteTitle();
                    isOver = false;
                }
            }else if(!isOut){
                deleteTitle();
                isOut = true;
            }
            
        });
        return firstUi;
    }
    addTitle(width,rightleft,topbot,style,overObject,getString){
        ///Относительные ли UI координаты
        let that = this;
        let responseTime = 500;
        let closeTime = 20000;
        let isOver = false;
        let isOut = true;
        let startOverTime = null;
        ////0 Создаем массив для текстов
        /////1 Разбиваем на части строку - получаем массив
        /////2 Проверяем каждое слово с искомым массивом слов, создаем текст каждого слова в нужном стиле
        let mainWidget = null;
        function createTitle(string){
            if(mainWidget == null){
                mainWidget = that.GUI.addUI(that.game.input.mousePointer.x,that.game.input.mousePointer.y,width,0,null,null,10);
                let text = mainWidget.addText(1,true,0,0,getString(),style,0,false);
                
                mainWidget.width = text.width + 10;
                mainWidget.height = text.height + 10;
                text.x = mainWidget.x + mainWidget.width/2;
                text.y = mainWidget.y + mainWidget.height/2;
                let uiAlign = getAlign();
                move(uiAlign.rl,uiAlign.tb);
                mainWidget.addBackgroundRoundedRect(0,true,true,0x000000,that.hexToOct(style.fill),1,0,1,15,false);
                mainWidget.addTween(['texts'],{ alpha: 1 },500);
                mainWidget.addTween(['graphics'],{ alpha: 0.8 },500);
                function getAlign(){
                    let rl = 'right';
                    let tb = 'top';
                    if( (that.game.input.mousePointer.x + mainWidget.width) > that.game.width){
                        rl = 'left';
                    }
                    ///Сдесь нужно отталкиваться от верхней точки либо иры либо камеры
                    if((that.game.input.mousePointer.y - mainWidget.height) < 0){
                        tb = 'bot';
                    }
                    return {rl:rl,tb:tb};
                }
                function move(rl,tb){
                    if(rl == 'right'){
                        if(tb == 'top'){
                            mainWidget.move(that.game.input.mousePointer.x,that.game.input.mousePointer.y - mainWidget.height);

                        }else if(tb == 'bot'){
                            mainWidget.move(that.game.input.mousePointer.x,that.game.input.mousePointer.y);

                        }

                    }else if(rl == 'left'){

                        if(tb == 'top'){
                            mainWidget.move(that.game.input.mousePointer.x - mainWidget.width,that.game.input.mousePointer.y - mainWidget.height);

                        }else if(tb == 'bot'){
                            mainWidget.move(that.game.input.mousePointer.x - mainWidget.width,that.game.input.mousePointer.y);

                        }
                    }
                };
                let lastCursorX = that.game.input.mousePointer.x;
                let lastCursorY = that.game.input.mousePointer.y;
                mainWidget.addUpdate(function(){
                    //Если положение курсора изменилось - двигаем тайтл
                    if((lastCursorX !== that.game.input.mousePointer.x) || (lastCursorY != that.game.input.mousePointer.y)){
                        lastCursorX = that.game.input.mousePointer.x;
                        lastCursorY = that.game.input.mousePointer.y;
                        uiAlign = getAlign();
                        move(uiAlign.rl,uiAlign.tb);
                    }
                })
                
            }
        }
        function deleteTitle(){
            if(mainWidget !== null){
                
                mainWidget.delete();
                mainWidget = null;
            }
        }
        function ifLeftTime(time){
            if( (new Date() - startOverTime) > time)
                return true;
            else return false
        }
        
        if(overObject.input.enabled == true){
            overObject.events.onInputOver.add(function(){
                isOver = true;
                startOverTime = new Date();
                isOut = false;
            },this);
            overObject.events.onInputOut.add(function(){
                isOver = false;
            },this);
        }
        that.onUpdate.push(function(){
            
           if(overObject.input.isDragged){
                    deleteTitle();
            }
            if(isOver){
                ///Если навели и время ответа прошло
                if( ifLeftTime(responseTime) ){
                    createTitle(getString())
                }
                //Если прошло слишком много времени удаляем ui
                if( ifLeftTime(closeTime) ){
                    deleteTitle();
                    isOver = false;
                }
            }else if(!isOut){
                deleteTitle();
                isOut = true;
            }
            
        });
        return mainWidget;
    }
    /*
    addTitle(layer,textWidth,text,obj,rightleft,topbot){
        
        if(obj.inputEnabled == true){
            let ui = null;
            obj.events.onInputOver.add(function(){
                
                    if(ui == null){
                        ui = this.addUI(layer,false,obj.x,obj.y,0,0,3);
                        
                        let textSize = 12;
                        let str = ui.addText(1,true,0,0,text,textSize,1,false);
                        str.wordWrapWidth = textWidth;
                        str.fill = '#FFE4C4';
                        //str.fill = '#FFFF00';
                        str.anchor.setTo(0.5,0.5);
                        ui.width = str.width + 10;
                        ui.height = str.height + 10;
                        let x,y;
                        if(rightleft == 'right'){
                            x = obj.x;
                        }else if(rightleft == 'left'){
                            x = obj.x - ui.width;
                        }
                        if(topbot == 'top'){
                            y = obj.y - ui.height;
                        }else if(topbot == 'bot'){
                            y = obj.y;
                        }
                        ui.x = x;
                        ui.y = y;
                        str.x = ui.x+ui.width/2;
                        str.y = ui.y+ui.height/2;
                        ui.addBackgroundRect(0,true,true,0x000000,0x2F4F4F,1,0.7,1,false);
                    }
                setTimeout(function(){
                   if(ui != null){
                        ui.delete();
                        ui = null;
                    } 
                },2000);
               
                
            },this);
            obj.events.onInputOut.add(function(){
                if(ui != null){
                    ui.delete();
                    ui = null;
                }
                
            },this);
       
        }
        
      
    }
    */
    ///Добавить Тайтл с динамическим текстом
    addDynamycTitle(layer,textWidth,obj,rightleft,topbot,style,getTextFunction){
        if(obj.inputEnabled == true){
            let that = this;
            let updatedText = '';
            let str = null;
            let ui = null;
            let responseTime = 500;
            let closeTime = 20000;
            let isOver = false;
            let startOverTime = null;
            function createTitle(){
                if((ui == null) && (!obj.input.isDragged)){
                    let x,y;
                    if(that.fixedToCamera){
                        x = obj.x - that.game.camera.x;
                        y = obj.y - that.game.camera.y;
                    }else{
                        x = obj.x;
                        y = obj.y;
                    }
                    ui = that.GUI.addUI(x,y,0,0,null,1,10);
                    //ui = that.addUI(layer,false,x,y,0,0,3);
                    let textSize = 12;
                    str = ui.addTextWithStyle(1,true,0,0,getTextFunction(),style,1,false);
                    str.wordWrapWidth = textWidth;
                    //str.fill = '#FFE4C4';
                    //str.fill = '#FFFF00';
                    str.anchor.setTo(0.5,0.5);
                    ui.width = str.width + 10;
                    ui.height = str.height + 10;
                    let newX,newY;
                    if(rightleft == 'right'){
                        newX = x;
                    }else if(rightleft == 'left'){
                        newX = x - ui.width;
                    }
                    if(topbot == 'top'){
                        newY = y - ui.height;
                    }else if(topbot == 'bot'){
                        newY = y;
                    }
                    ui.x = newX;
                    ui.y = newY;
                    str.x = ui.x+ui.width/2;
                    str.y = ui.y+ui.height/2;
                    ui.addBackgroundRect(0,true,true,0x000000,0x000000,1,0.7,1,false);
                    return true
                }else return false;
            }
            function deleteTitle(){
                if(ui != null){
                    ui.delete();
                    ui = null;
                }
                startOverTime = null;
            }
            function ifLeftTime(time){
                if( (new Date() - startOverTime) > time)
                    return true;
                else return false
            }
            
            obj.events.onInputOver.add(function(){
                isOver = true;
                startOverTime = new Date();
            },this);
            obj.events.onInputOut.add(function(){
                isOver = false;
            },this);
            that.onUpdate.push(function(){
                if(obj.input.isDragged){
                    deleteTitle();
                }
                if(isOver){
                    ///Если навели и время ответа прошло
                    if( ifLeftTime(responseTime) ){
                        if(createTitle()){
                            str.setText(getTextFunction());
                        }
                    }
                    //Если прошло слишком много времени удаляем ui
                    if( ifLeftTime(closeTime) ){
                        deleteTitle();
                        isOver = false;
                    }
                }else{
                    deleteTitle();
                }
            });
       
        }
        
      
    }
    /*
    ///Добавить Тайтл с динамическим текстом
    addDynamycTitleWithColors(layer,textWidth,obj,rightleft,topbot,style,wordTypes,getTextFunction){
        if(obj.inputEnabled == true){
            let that = this;
            let updatedText = '';
            let str = null;
            let ui = null;
            let responseTime = 500;
            let closeTime = 20000;
            let isOver = false;
            let isOut = true;
            let startOverTime = null;
            function createTitle(){
                if((ui == null) && (!obj.input.isDragged)){
                    let x,y;
                    if(that.fixedToCamera){
                        x = obj.x - that.game.camera.x;
                        y = obj.y - that.game.camera.y;
                    }else{
                        x = that.game.input.mousePointer.x;
                        y = that.game.input.mousePointer.y;
                        //x = obj.x;
                        //y = obj.y;
                    }
                    ui = that.GUI.addUI(x,y,0,0,null,1,10);
                    //ui = that.addUI(layer,false,x,y,0,0,3);
                    let textSize = 12;
                    //addDifferentColorTextWithStyle
                    str = ui.addTextWithCustomStyle(1,true,0,0,getTextFunction(),wordTypes,style,'center',1,false);
                    //str = ui.addText(1,true,0,0,getTextFunction(),textSize,1,false);
                    str.wordWrapWidth = textWidth;
                    //str.fill = '#FFE4C4';
                    //str.fill = '#FFFF00';
                    //str.x = Math.round(str.x/2);
                    //str.y = Math.round(str.y/2);
                    //str.anchor.setTo(0.5,0.5);
                    ui.width = str.width + 30;
                    ui.height = str.height + 10;
                    let newX,newY;
                    if(rightleft == 'right'){
                        newX = x;
                    }else if(rightleft == 'left'){
                        newX = x - ui.width;
                    }
                    if(topbot == 'top'){
                        newY = y - ui.height;
                    }else if(topbot == 'bot'){
                        newY = y;
                    }
                    ui.move(newX,newY);
                    //ui.x = newX;
                    //ui.y = newY;
                    
                    
                    str.x = Math.round(ui.x+ui.width/2 - str.width/2);
                    str.y = Math.round(ui.y+ui.height/2 - str.height/2);
                    
                    let bg = ui.addBackgroundRoundedRect(0,true,true,0x000000,that.hexToOct(style.fill),1,0.8,1,15,false);
                    //let bg = ui.addBackgroundRect(0,true,true,0x000000,that.hexToOct(style.fill),1,0.7,1,false);
                    bg.alpha = 0;
                    str.alpha = 0;
                    game.add.tween(bg).to( { alpha: 0.8 }, 500, "Linear", true);
                    game.add.tween(str).to( { alpha: 1 }, 500, "Linear", true);
                    let oldWidth = str.width;
                    let oldHeight = str.height;
                    ui.addUpdate(function(){
                        ///Если размер строки менялся
                        if((oldWidth !== str.width) || (oldHeight !== str.height)){
                            ///Если изменился
                            oldWidth = str.width;
                            oldHeight = str.height;
                            
                            ui.width = str.width + 30;
                            ui.height = str.height + 10;
                        }
                        //Если положение курсора изменилось - двигаем тайтл
                        if((ui.x !== that.game.input.mousePointer.x) || (ui.y != that.game.input.mousePointer.y)){
                            if(rightleft == 'right'){
                                if(topbot == 'top'){
                                    ui.move(that.game.input.mousePointer.x,that.game.input.mousePointer.y - ui.height);
                                    // ui.y = that.game.input.mousePointer.y - ui.height;
                                }else if(topbot == 'bot'){
                                    ui.move(that.game.input.mousePointer.x,that.game.input.mousePointer.y);
                                    //ui.y = that.game.input.mousePointer.y;
                                }
                                //ui.x = that.game.input.mousePointer.x;
                            }else if(rightleft == 'left'){
                            
                                //ui.x = that.game.input.mousePointer.x - ui.width;
                                if(topbot == 'top'){
                                    ui.move(that.game.input.mousePointer.x - ui.width,that.game.input.mousePointer.y - ui.height);
                                    // ui.y = that.game.input.mousePointer.y - ui.height;
                                }else if(topbot == 'bot'){
                                    ui.move(that.game.input.mousePointer.x - ui.width,that.game.input.mousePointer.y);
                                    //ui.y = that.game.input.mousePointer.y;
                                }
                            }
                          
                            //ui.x = that.game.input.mousePointer.x;
                            //ui.y = that.game.input.mousePointer.y;
                            
                            
                            //str.x = Math.round(ui.x+ui.width/2 - str.width/2);
                            //str.y = Math.round(ui.y+ui.height/2 - str.height/2);
                        }
                        
                    })
                    return true
                }else return false;
            }
            function deleteTitle(){
                if(ui != null){
                    ui.delete();
                    ui = null;
                }
                startOverTime = null;
            }
            function ifLeftTime(time){
                if( (new Date() - startOverTime) > time)
                    return true;
                else return false
            }
            
            obj.events.onInputOver.add(function(){
                isOver = true;
                startOverTime = new Date();
                isOut = false;
            },this);
            obj.events.onInputOut.add(function(){
                isOver = false;
            },this);
            that.onUpdate.push(function(){
           if(obj.input.isDragged){
                    deleteTitle();
            }
            if(isOver){
                ///Если навели и время ответа прошло
                if( ifLeftTime(responseTime) ){
                    createTitle()
                }
                //Если прошло слишком много времени удаляем ui
                if( ifLeftTime(closeTime) ){
                    deleteTitle();
                    isOver = false;
                }
            }else if(!isOut){
                deleteTitle();
                isOut = true;
            }
            
        });
       
        }
        
      
    }
    addDynamycTitle2(layer,textWidth,obj,rightleft,topbot,style,getTextFunction){
        let that = this;
        let updatedText = '';
        let str = null;
        if(obj.inputEnabled == true){
            let ui = null;
            let timer = null;
            let owerTime = 500;
            let isOver = false;
            let lifeTime = 10000;
            obj.events.onInputOver.add(function(){
                isOver = true;
                timer = setTimeout(function(){
                    if(ui == null){
                        let x,y;
                        if(that.fixedToCamera){
                            x = obj.x - that.game.camera.x;
                            y = obj.y - that.game.camera.y;
                        }else{
                            x = obj.x;
                            y = obj.y;
                        }
                        ui = that.addUI(layer,false,x,y,0,0,3);
                        let textSize = 12;
                        str = ui.addTextWithStyle(1,true,0,0,getTextFunction(),style,1,false);
                        //str = ui.addText(1,true,0,0,getTextFunction(),textSize,1,false);
                        str.wordWrapWidth = textWidth;
                        //str.fill = '#FFE4C4';
                        //str.fill = '#FFFF00';
                        str.anchor.setTo(0.5,0.5);
                        ui.width = str.width + 10;
                        ui.height = str.height + 10;
                        let newX,newY;
                        
                        if(rightleft == 'right'){
                            newX = x;
                        }else if(rightleft == 'left'){
                            newX = x - ui.width;
                        }
                        if(topbot == 'top'){
                            newY = y - ui.height;
                        }else if(topbot == 'bot'){
                            newY = y;
                        }
                        ui.x = newX;
                        ui.y = newY;
                        str.x = ui.x+ui.width/2;
                        str.y = ui.y+ui.height/2;
                        
                        ui.addBackgroundRect(0,true,true,0x000000,that.hexToOct(style.fill),1,0.7,1,false);
                    }
                setTimeout(function(){
                   if(ui != null){
                        ui.delete();
                        ui = null;
                    } 
                },lifeTime);
            },owerTime);
               
                
            },this);
            obj.events.onInputOut.add(function(){
                isOver = false;
                if(ui != null){
                    ui.delete();
                    ui = null;
                }
                if(timer != null){
                    clearTimeout(timer);
                }
                
            },this);
            
            that.onUpdate.push(function(){
                if(ui !== null){
                    getTextFunction();
                }
            });
       
        }
        
      
    }
    addNewDynamycTitle(layer,textWidth,obj,rightleft,topbot,getTextFunction){
        let that = this;
        let ui = null;
        let text = null;
        let bg = null;
        let isOver = false;
        let startOver = null;
        if(obj.inputEnabled == true){
            obj.events.onInputOver.add(function(){
                isOver = true;
                startOver = new Date();
            });
            obj.events.onInputOut.add(function(){
                isOver = false;
                startOver = null;
            });
                                       
        }
        this.onUpdate.push(function(){
            ///Если навели
            if(isOver){
                console.log(new Date() - startOver)
                //Если навели более N времени
                if((new Date() - startOver) > 500){
                    ///Получаем строку
                    let string = getTextFunction();
                    ///Получаем координаты объекта
                    let x,y;
                    if(that.fixedToCamera){
                        x = obj.x - that.game.camera.x;
                        y = obj.y - that.game.camera.y;
                    }else{
                        x = obj.x;
                        y = obj.y;
                    }
                    ///Добавляем UI
                    if(ui == null){
                        ui = that.addUI(layer,false,x,y,0,0,3);
                        ///Создаем текст
                        if(text == null){
                            text = ui.addText(1,true,0,0,string,14,1,false);
                            text.anchor.setTo(0.5,0.5);
                            text.wordWrapWidth = textWidth;
                        }
                        
                    }
                    ///Обновляем размеры и координаты UI
                    //ui.x = x;
                    //ui.y = y;
                    ui.width = text.width + 10;
                    ui.height = text.height + 10;
                    
                    let newX,newY;
                    if(rightleft == 'right'){
                        newX = x;
                    }else if(rightleft == 'left'){
                        newX = x - ui.width;
                    }
                    if(topbot == 'top'){
                        newY = y - ui.height;
                    }else if(topbot == 'bot'){
                        newY = y;
                    }
                    ui.x = x;
                    ui.y = y;
                    /////САМЕРА ОФСЕТ РАЗОБРАТЬСЯ
                    if(that.fixedToCamera){
                        text.cameraOffset.x = ui.x+ui.width/2;
                        text.cameraOffset.y = ui.y+ui.height/2;
                    }else{
                        text.x =  ui.x+ui.width/2;
                        text.y = ui.y+ui.height/2;
                    }
                    text.setText(string);
                    console.log(ui.x,ui.y,text.x,text.y)
                    if(bg == null){
                        bg = ui.addBackgroundRect(0,true,true,0x000000,0x2F4F4F,1,0.7,1,false);
                    }else{
                        bg.x = ui.x;
                        bg.y = ui.y;
                        //bg.width = ui.width;
                        //bg.height = ui.height;
                    }
                    
                    
                    
                }
            }else{
                if(ui !== null){
                    ui.delete();
                    ui = null;
                    text = null; 
                    bg = null;
                }
                
            }
        });      
      
    }
    */
    addScrollTextAreaHtml(layer,relativCoord,x,y,w,h,textSize,getTextFunction){
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        let ui = this.addUI(layer,relativCoord,coordX,coordY,w,h,1);
        let num = this.domElements.push(document.createElement('div'));
        let div = this.domElements[num - 1];
        //div.setAttribute('id','newdiv');
        div.style.overflowY = 'scroll';
        div.style.overflowX = 'hidden';
        div.style.height = ""+h+"px";
        div.style.position = 'fixed';
        div.style.width = ""+w+"px";
        div.style.top = ""+coordY+"px";
        div.style.left = ""+coordX+"px";
        div.style.wordWrap = 'break-word';
        //div.style.padding = "20px";
        //newdiv.style.overflow = 'scroll';
        document.body.appendChild(div);
        ui.onUpdate.push(function(){
            div.innerHTML = getTextFunction();
            //text.setText(newText);
            
        });
        return this.domElements[num - 1];
        
    }
    addScrollTextArea(layer,relativCoord,x,y,w,h,string,style,wButtons,hButtons,wSlider,hSlider,upButtonKey,downButtonKey,sliderKey){
        let mainUi = this.addUI(layer,relativCoord,x,y,w,h,5);
        let bgMain = mainUi.addBackgroundRect(0,false,true,0xFFFFFF,0x000000,1,0.1,1,false);
        let ui = mainUi.addUI(1,true,mainUi.width*0.05,mainUi.height*0.05,mainUi.width*0.8,mainUi.height*0.9,5);
        //let bg = ui.addBackgroundRect(0,false,true,0xFFFFFF,0x000000,2,0.1,1,false);
        
        let text = ui.addText(1,true,0,0,string,style,1,false);
        text.anchor.setTo(0.0,0.0);

        
        //let bg = ui.addBackgroundRect(0,true,true,GAMEPARAMS.UI.rectFillCalor,GAMEPARAMS.UI.borderFillCalor,1,1,1,true);
        
        //let text = ui.addBitMapText(1,true,0,0,string,font,textSize,1,false);
        text.wordWrapWidth = ui.width - 5;
        let num = this.graphics.push(this.game.add.graphics(ui.x,ui.y));
        let mask = this.graphics[num -1];
        //let mask = game.add.graphics(ui.x, ui.y);
        mask.beginFill(0xffffff);
        mask.drawRect(0, 0, ui.width , ui.height);
        text.mask = mask;
        mask.fixedToCamera = this.fixedToCamera;
        
        function canScrollUp(){
            if((text.y - 10)  > ui.y){
                return false
            }else return true;
        };
        function canScrollDown(){
            if((text.y + text.height + 10) < ( ui.y + ui.height)){
                return false
            }else return true;
        };
        
        let upScrollButton,downScrollButton;
        upScrollButton = mainUi.addLongClickButton(2,true,mainUi.width - 20,20,wButtons,hButtons,upButtonKey,function(){
           if(canScrollUp()){
               slider.percent -= 5;
           }
   
        },function(progress){
            if(canScrollUp()){
                slider.percent -= 5*progress;
         
           }

        });
         downScrollButton = mainUi.addLongClickButton(2,true,mainUi.width - 20,mainUi.height - 20,wButtons,hButtons,downButtonKey,function(){
             if(canScrollDown()){
                 slider.percent += 5;
            
           }
        },function(progress){
             if(canScrollDown()){
                 slider.percent += 5*progress;
            
           }
        });
        let canDragSlider = true;    
        let slider = mainUi.addVerticalSlider(4,false,upScrollButton.y + (upScrollButton.height/2),downScrollButton.y - (downScrollButton.height/2),upScrollButton.x,wSlider,hSlider,5,0x000000,0.2,sliderKey,function(){
           return canDragSlider;
        });
        let lastPercent  = 0;
        ui.onUpdate.push(function(){
            
            if(text.height < ui.height){
                canDragSlider = false;
            }else{
                canDragSlider = true;
            }
           
            let newY = (ui.y) - (((text.height - ui.height)/100)*slider.percent);
            if(lastPercent <  slider.percent){
                if(canScrollDown()){
                    text.y = newY;
                }else{
                    slider.percent = lastPercent;
                }
            }
            if(lastPercent >  slider.percent){
                if(canScrollUp()){
                    text.y = newY;
                }else{
                    slider.percent = lastPercent;
                }
            }
            lastPercent = slider.percent;
        });
        return text;
        
    }
    addScrollTextsArea(layer,relativCoord,x,y,w,h,style,wButtons,hButtons,wSlider,hSlider,upButtonKey,downButtonKey,sliderKey,getStringsArrayFunction,onStringClickFunction){
        let mainUi = this.addUI(layer,relativCoord,x,y,w,h,5);
        let bgMain = mainUi.addBackgroundRect(0,false,true,0xFFFFFF,0x000000,2,0.1,1,false);
        let originUI = mainUi.addUI(1,true,mainUi.width*0.05,mainUi.height*0.05,mainUi.width*0.8,mainUi.height*0.9,5);
        let ui = originUI.addUI(1,true,0,0,originUI.width,originUI.height,5);
        //ui.addBackgroundRect(0,false,true,0xFFFFFF,0x000000,2,0.1,1,false);
        
        let num = this.graphics.push(this.game.add.graphics(ui.x,ui.y));
        let mask = this.graphics[num -1];
        //let mask = game.add.graphics(ui.x, ui.y);
        mask.beginFill(0xffffff);
        mask.drawRect(0, 0, ui.width , ui.height);
        let upScrollButton,downScrollButton;
        function canScrollUp(){
            if((text.y - 10)  > ui.y){
                return false
            }else return true;
        };
        function canScrollDown(){
            if((text.y + text.height + 10) < ( ui.y + ui.height)){
                return false
            }else return true;
        };
        upScrollButton = mainUi.addLongClickButton(2,true,mainUi.width - 20,20,wButtons,hButtons,upButtonKey,function(){
            ui.y --;
            /*
           if(canScrollUp()){
               slider.percent -= 5;
           }
           */
   
        },function(progress){
            ui.y --;
            /*
            if(canScrollUp()){
                slider.percent -= 5*progress;
         
           }
           */

        });
        function getTextsHeight(){
            if(texts.length >= 1){
                return (texts[0].y - texts[0].height/2) - (texts[texts.length - 1].y + texts[texts.length - 1].height/2)
            }else{
                return 0;
            }
             
        }
         downScrollButton = mainUi.addLongClickButton(2,true,mainUi.width - 20,mainUi.height - 20,wButtons,hButtons,downButtonKey,function(){
             ui.y ++;
             /*
             if(canScrollDown()){
                 slider.percent += 5;
            
           }
           */
        },function(progress){
             ui.y ++;
             /*
             if(canScrollDown()){
                 slider.percent += 5*progress;
            
           }
           */
        });
        /*
        let canDragSlider = true;    
        let slider = mainUi.addVerticalSlider(4,false,upScrollButton.y + (upScrollButton.height/2),downScrollButton.y - (downScrollButton.height/2),upScrollButton.x,wSlider,hSlider,5,0x000000,0.2,sliderKey,function(){
           return canDragSlider;
        });
        */
        let texts = [];
        let lastStrings = null;
        let textInterval = 15;
        ui.onUpdate.push(function(){
            
            let strings = getStringsArrayFunction();
            if(JSON.stringify(lastStrings) !== JSON.stringify(strings)){
                
                for(let i in texts){
                    texts[i].destroy();
                }
                
                let count = 0;
                for(let i in strings){
                    //addTextWithStyle(layer,relativCoord,x,y,text,style,alpha,inputEnable)
                    
                    let num = texts.push(ui.addText(1,true,0,count*textInterval,strings[i],style,1,true));
                    let text = texts[num -1];
                    text.anchor.setTo(0.0,0.0);
                    text.events.onInputDown.add(function(){
                        onStringClickFunction(text);
                    });
                    text.mask = mask;
                    
                    count++;
                }
                lastStrings = strings;
            }
            
            ui.height = texts.length*textInterval;
            
            
            if(ui.y + ui.height < originUI.y + originUI.height){
                ui.y  = originUI.y + originUI.height - ui.height;
            }
            if(ui.y > (originUI.y)){
                ui.y = originUI.y;
            }
            
        });
        
    }
    addScrollAreaWithUsableTextObjects(layer,relativCoord,x,y,w,h,string,style,wButtons,hButtons,wSlider,hSlider,upButtonKey,downButtonKey,sliderKey){
        let mainUi = this.addUI(layer,relativCoord,x,y,w,h,5);
        let bgMain = mainUi.addBackgroundRect(0,false,true,0xFFFFFF,0x000000,2,0.1,1,false);
        let ui = mainUi.addUI(1,true,mainUi.width*0.05,mainUi.height*0.05,mainUi.width*0.8,mainUi.height*0.9,5);
        //let bg = ui.addBackgroundRect(0,false,true,0xFFFFFF,0x000000,2,0.1,1,false);
        
        let text = ui.addUpdateTextWithStyle(1,true,0,0,string,style,1,false);

        
        //let bg = ui.addBackgroundRect(0,true,true,GAMEPARAMS.UI.rectFillCalor,GAMEPARAMS.UI.borderFillCalor,1,1,1,true);
        
        //let text = ui.addBitMapText(1,true,0,0,string,font,textSize,1,false);
        text.wordWrapWidth = ui.width - 5;
        
        let mask = game.add.graphics(ui.x, ui.y);
        mask.beginFill(0xffffff);
        mask.drawRect(0, 0, ui.width , ui.height);
        text.mask = mask;
        mask.fixedToCamera = this.fixedToCamera;
        
        function canScrollUp(){
            if((text.y - 10)  > ui.y){
                return false
            }else return true;
        };
        function canScrollDown(){
            if((text.y + text.height + 10) < ( ui.y + ui.height)){
                return false
            }else return true;
        };
        
        let upScrollButton,downScrollButton;
        upScrollButton = mainUi.addLongClickButton(2,true,mainUi.width - 20,20,wButtons,hButtons,upButtonKey,function(){
           if(canScrollUp()){
               slider.percent -= 5;
           }
   
        },function(progress){
            if(canScrollUp()){
                slider.percent -= 5*progress;
         
           }

        });
         downScrollButton = mainUi.addLongClickButton(2,true,mainUi.width - 20,mainUi.height - 20,wButtons,hButtons,downButtonKey,function(){
             if(canScrollDown()){
                 slider.percent += 5;
            
           }
        },function(progress){
             if(canScrollDown()){
                 slider.percent += 5*progress;
            
           }
        });
        let canDragSlider = true;    
        let slider = mainUi.addVerticalSlider(4,false,upScrollButton.y + (upScrollButton.height/2),downScrollButton.y - (downScrollButton.height/2),upScrollButton.x,wSlider,hSlider,5,0x000000,0.2,sliderKey,function(){
           return canDragSlider;
        });
        let lastPercent  = 0;
        ui.onUpdate.push(function(){
            
            if(text.height < ui.height){
                canDragSlider = false;
            }else{
                canDragSlider = true;
            }
           
            let newY = (ui.y) - (((text.height - ui.height)/100)*slider.percent);
            if(lastPercent <  slider.percent){
                if(canScrollDown()){
                    text.y = newY;
                }else{
                    slider.percent = lastPercent;
                }
            }
            if(lastPercent >  slider.percent){
                if(canScrollUp()){
                    text.y = newY;
                }else{
                    slider.percent = lastPercent;
                }
            }
            lastPercent = slider.percent;
        });
        return text;
        
    }
    ///Создание скроллящегося текста
    addScrollBitMapTextArea(layer,relativCoord,x,y,w,h,string,textSize,font,wButtons,hButtons,wSlider,hSlider,upButtonKey,downButtonKey,sliderKey){
   
        let mainUi = this.addUI(layer,relativCoord,x,y,w,h,5);
        let ui = mainUi.addUI(1,true,mainUi.width*0.05,mainUi.height*0.1,mainUi.width*0.8,mainUi.height*0.8,5);
        //let bg = ui.addBackgroundRect(0,true,true,GAMEPARAMS.UI.rectFillCalor,GAMEPARAMS.UI.borderFillCalor,1,1,1,true);
        
        let text = ui.addBitMapText(1,true,0,0,string,font,textSize,1,false);
        text.wordWrapWidth = ui.width - ui.width*0.2;
        let mask = game.add.graphics(ui.x, ui.y);
        mask.beginFill(0xffffff);
        mask.drawRect(0, 0, ui.width, ui.height*1.05);
        text.mask = mask;
        mask.fixedToCamera = this.fixedToCamera;
        function canScrollUp(){
            if((text.y - 10)  > ui.y){
                return false
            }else return true;
        };
        function canScrollDown(){
            if((text.y + text.height + 10) < ( ui.y + ui.height)){
                return false
            }else return true;
        };
        
        let upScrollButton,downScrollButton;
        upScrollButton = mainUi.addLongClickButton(2,true,mainUi.width - 20,20,wButtons,hButtons,upButtonKey,function(){
           if(canScrollUp()){
               slider.percent -= 5;
           }
   
        },function(progress){
            if(canScrollUp()){
                slider.percent -= 5*progress;
         
           }

        });
         downScrollButton = mainUi.addLongClickButton(2,true,mainUi.width - 20,mainUi.height - 20,wButtons,hButtons,downButtonKey,function(){
             if(canScrollDown()){
                 slider.percent += 5;
            
           }
        },function(progress){
             if(canScrollDown()){
                 slider.percent += 5*progress;
            
           }
        });
            
        let slider = mainUi.addVerticalSlider(4,false,upScrollButton.y + (upScrollButton.height/2),downScrollButton.y - (downScrollButton.height/2),upScrollButton.x,wSlider,hSlider,5,0x000000,0.2,sliderKey)
        let lastPercent  = 0;
        ui.onUpdate.push(function(){
           
            let newY = (ui.y) - (((text.height - ui.height)/100)*slider.percent);
            if(lastPercent <  slider.percent){
                if(canScrollDown()){
                    text.y = newY;
                }else{
                    slider.percent = lastPercent;
                }
            }
            if(lastPercent >  slider.percent){
                if(canScrollUp()){
                    text.y = newY;
                }else{
                    slider.percent = lastPercent;
                }
            }
            lastPercent = slider.percent;
        });
        return text;
        
    }
    addTextFieldHtml(layer,relativCoord,x,y,w,h,textSize,placeholder,maxlength,size,type){
        /*
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+(x - (w/2));
            coordY = this.y+(y - (h/2));
        }
        else{
            coordX = (x - (w/2));
            coordY = (y - (h/2));
        }
        */
        let ui = this.addUI(layer,relativCoord,x,y,w,h,1);
        let num = this.domElements.push(document.createElement('input'));
        let div = this.domElements[num - 1];
        div.setAttribute('type',type);
        div.setAttribute('maxlength',maxlength);
        div.setAttribute('size',size);
        div.setAttribute('placeholder',placeholder);
        //div.setAttribute('disabled', 'disabled');
        div.style.textAlign = "center";
        div.style.fontSize = ""+textSize+"px";
        div.style.height = ""+h+"px";
        div.style.position = 'absolute';
        div.style.width = ""+w+"px";
        div.style.top = ""+(ui.y -( ui.height/2))+"px";
        div.style.left = ""+(ui.x -( ui.width/2))+"px";
        div.style.backgroundColor = 'transparent';
        div.style.border = 'none';
        //div.style.padding = "20px";
        
        //newdiv.style.overflow = 'scroll';
        document.body.appendChild(div);
        /*
        ui.onUpdate.push(function(){
            div.innerHTML = getTextFunction();
            //text.setText(newText);
            
        });
        */
        /*
        this.addUpdate(function(){
          
            
        });
        */
        return this.domElements[num - 1];
        
    }
    addTween(types,tween,time){
        for(let i in types){
            switch(types[i]){
                case 'texts':
                    for(let i in this.texts){
                        this.game.add.tween(this.texts[i]).to(tween , time, "Linear", true);
                    }
                    break;
                case 'images':
                    for(let i in this.images){
                        this.game.add.tween(this.images[i]).to(tween , time, "Linear", true);
                    }
                    break;
                case 'graphics':
                    for(let i in this.graphics){
                        this.game.add.tween(this.graphics[i]).to(tween , time, "Linear", true);
                    }
            }
         
        }
        for(let i in this.UIArray){
            this.UIArray[i].addTween(types,tween,time);
        }
        
    }
    addSound(key,loop,volume){
        let num = this.audio.push(this.game.add.audio(key,volume,loop));
        return this.audio[num - 1];
    }
    move(newX,newY){
        //console.log('new:',newX,newY)
        this.lastX = this.x;
        this.lastY = this.y;
        this.x = newX;
        this.y = newY;
    }
    wasMoved(){
        //console.log(this.lastX,this.x,this.lastY,this.y)
        if((this.lastX !== this.x) || (this.lastY !== this.y)){
            return true;
        }else return false;
    }
    delete(){
        this.clear();
        this.setDeleted();
        
        
    }
    setDeleted(){
        this.deleted = true;
    }
    isDeleted(){
        return this.deleted;
    }
    clear(){
        ///Удаление функций ПРИ УДАЛЕНИИ
        for(let fn in this.onDelete){
            this.onDelete[fn]();
        }
        /////////////////////////////////
        
        
        for(let bk in this.backgrounds){
            
            this.backgrounds[bk].destroy();
            
            //
        }
        this.backgrounds.splice(0,this.backgrounds.length);
        
        for(let tx in this.texts){
            
            this.texts[tx].destroy();
            
            //
        }
        this.texts.splice(0,this.texts.length);
        
        for(let i in this.images){
            this.images[i].destroy();
            
            //
        }
        this.images.splice(0,this.images.length);
        
        for(let b in this.buttons){
            this.buttons[b].destroy();
        }
        this.buttons.splice(0,this.buttons.length);
        
        for(let s in this.sliders){
            delete this.sliders[s];
        }
        this.sliders.splice(0,this.sliders.length);
        
        for(let f in this.inpuFields){
            this.inpuFields[f].destroy();
        }
        this.inpuFields.splice(0,this.inpuFields.length);
        
        for(let i in this.audio){
            this.audio[i].stop();
            this.audio[i].destroy();
        }
        this.audio.splice(0,this.audio.length);
        
        for(let g in this.graphics){
            this.graphics[g].destroy();
        }
        this.graphics.splice(0,this.graphics.length);
        
        
        for(let de in this.domElements){
            this.domElements[de].remove();
        }
        this.domElements.splice(0,this.domElements.length);
        
        
        this.onDelete.splice(0,this.onDelete.length);
        
        for(let i in this.onUpdate){
            delete this.onUpdate[i];
        }
        this.onUpdate.splice(0,this.onUpdate.length);
        
        for(let i in this.onCreate){
            delete this.onCreate[i];
        }
        this.onCreate.splice(0,this.onCreate.length);
        ////
        for(let ui in this.UIArray){
            this.UIArray[ui].delete();
        }  
        this.UIArray.splice(0,this.UIArray.length);
    }
    update(){
        
        if(this.fixedToCamera){
            this.currentX = this.game.camera.x + this.x;
            this.currentY = this.game.camera.y + this.y;
        }
        for(let i in this.onUpdate){
            this.onUpdate[i]();
        }
        for(let i = 0; i < this.UIArray.length; i++){
            ///Если UI отмечен для удаления
            if(this.UIArray[i].isDeleted()){
                this.UIArray.splice(i,1);
                i--;
            }else
                this.UIArray[i].update();
        }
        
        if(this.sortByY === true){
            for(let i in this.layers){
                this.layers[i].sort('y', Phaser.Group.SORT_ASCENDING);
            }
        }
        
        
        
    }
    
    scale(x,y){
        this.layerGroup.scale.set(x,y);
    }
    ///Для преобразования цветов типо "#FFDAB9" в 0xFFDAB9
    hexToOct(hex){
        let newStr = hex.slice(1);
        newStr = '0x'+newStr;
        return newStr;
        
    }
    
    
    
    
    
    
    
    
    //////////////////////СТАРОЕ
    /*
    addTextWithCustomStyle(layer,relativCoord,x,y,text,wordTypes,style,align,alpha,inputEnable){
        let that = this;
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        let num = this.texts.push(this.game.add.text(coordX,coordY,text,style,this.layers[layer]));
        let textSprite = this.texts[num - 1];

        textSprite.alpha = alpha;

        function isLetter(str) {
            if(str.length === 1){
                if(str.match(/[a-zA-zа-яА-Я0-9]/i)){
                    return true;
                }
            }
            return false;

        }
        if(inputEnable){
            textSprite.inputEnabled = true;
            textSprite.input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
        textSprite.fixedToCamera = this.fixedToCamera;
        let originWeightText = textSprite.fontWeight;
        let lastTextString = null;
        let lastText = text;
        let lastX = textSprite.x;
        let lastY = textSprite.y;
        this.addUpdate(function(){
            ///Если у нас изменился текст
            if(lastTextString !== textSprite.text){
  
                
                let textArray = textSprite.text.split(/[ .:;?!~,'"&|()<>{}\[\]\r\n/\\]+/);
         
                let textFormats = [];
                let fontWeightFormats = [];
                for(let i in wordTypes){
                    ///Определяем тип: одиночное слово или строка
                    ///Если после разбивания элементов более 1 значит это строка
                    if(!wordTypes[i].word) return false;
                    let wordArray = wordTypes[i].word.split(/[ .:;?!~,'"&|()<>{}\[\]\r\n/\\]+/);
                    if(! (wordArray.length > 1) ){
                        
                        
                        var count = 0;
                        let strNumbArray = [];
                        var pos = text.indexOf(wordTypes[i].word);

                        while (pos !== -1) {
                            //console.log(wordTypes[i])
                            ///Если последний сивол этого слова не последний символ текста, и если после последнего символа этого текста не пробел/точка/запятая и так далее - значит мы нашли часть слова в слове - ТАКОЕ ВЫДЕЛЯТЬ НЕ НАДО
                            ////Если есть символ сразу после слова
                            ///Если следущий символ это буква - тогда выделять не надо (ИБО будет выделена часть слова) (ил символ предыдущий начала слова)
                            if( !( (text[pos+wordTypes[i].word.length] && isLetter(text[pos+wordTypes[i].word.length]) ) || ( text[pos -1] && isLetter(text[pos -1]) ) ) ){
                                textFormats.push({color:wordTypes[i].color, wordNumb:pos,style:wordTypes[i].style,weight:wordTypes[i].weight});
                                textFormats.push({color:style.fill, wordNumb:pos+wordTypes[i].word.length,style:textSprite.fontStyle,weight:textSprite.fontWeight});
                            }
                            
                            //console.log(wordTypes[i].word,count,pos);
                            pos = text.indexOf(wordTypes[i].word, pos + 1); 
                            
                        }
                
                    ///Если есть пробел - это строка
                    }else{
              
                        let strNumb = text.indexOf(wordTypes[i].word);
                        ///Если он нашео такую строку
                        if(strNumb !== -1){
                            //console.log('s',strNumb)
                            //console.log('l',wordTypes[i].word.length)
                            ///Ставим Новый цвет
                            textFormats.push({color:wordTypes[i].color, wordNumb:strNumb,style:wordTypes[i].style,weight:wordTypes[i].weight});
                            ///Возвращаем старый
                            textFormats.push({color:style.fill, wordNumb:strNumb+wordTypes[i].word.length,style:textSprite.fontStyle,weight:textSprite.fontWeight});
            
                        } 
                    }   
                }
                ///Чистим цвета теста
                textSprite.clearColors();
                textSprite.clearFontValues();
                for(let i in textFormats){
                    ///Добавляем цвета текста
                    textSprite.addColor(textFormats[i].color, textFormats[i].wordNumb);
                    textSprite.addFontStyle(textFormats[i].style,textFormats[i].wordNumb);
                    textSprite.addFontWeight(textFormats[i].weight,textFormats[i].wordNumb);
                    ///addFontStyle (стиль, позиция) ///normal, italicили oblique
                    ///addFontWeight (стиль, позиция) ///normal, bold, bolder, lighter
                }
                //console.log(textSprite)
                lastTextString = textSprite.text;
            }
            
            
            
            if(relativCoord){
                if(align == 'center'){
                    textSprite.x = Math.round(that.x+that.width/2 - textSprite.width/2);
                    textSprite.y = Math.round(that.y+that.height/2 - textSprite.height/2);
                }else{
                    coordX = that.x+x;
                    coordY = that.y+y;
                    textSprite.x = Math.round(coordX - (textSprite.width)/2);
                    textSprite.y = Math.round(coordY - (textSprite.height)/2);
                }   
            }else{
                coordX = x;
                coordY = y;
                textSprite.x = Math.round(coordX - (textSprite.width)/2);
                textSprite.y = Math.round(coordY - (textSprite.height)/2);
            }
          
            
        });
        return textSprite;
    }
    
    
    addDifferentColorTextWithStyle(layer,relativCoord,x,y,text,wordTypes,style,alpha,inputEnable){
        let that = this;
        ///Относительные ли UI координаты
        let coordX,coordY;
        if(relativCoord){
            coordX = this.x+x;
            coordY = this.y+y;
        }
        else{
            coordX = x;
            coordY = y;
        }
        let num = this.texts.push(this.game.add.text(coordX,coordY,text,style,this.layers[layer]));
        
        //this.texts[num - 1].anchor.setTo(0.5, 0.5);
        this.texts[num - 1].alpha = alpha;
        //this.texts[num - 1].font = style.font;
        
        if(inputEnable){
            this.texts[num - 1].inputEnabled = true;
            this.texts[num - 1].input.priorityID = this.inputPriority + (layer/this.layersCount);
        }
        this.texts[num - 1].fixedToCamera = this.fixedToCamera;
        let lastTextString = null;
        this.addUpdate(function(){
            ///Если у нас изменился текст
            if(lastTextString !== that.texts[num - 1].text){
                let textArray = that.texts[num - 1].text.split(/(?:,| )+/);
                console.log(textArray)
                let textFormats = [];
                for(let i in wordTypes){

                    let wordNumb = 0;
                    let stop = false;
                    for(let j in textArray){
                        if(!stop){

                            if(textArray[j] === wordTypes[i].word){
                                console.log(wordTypes[i].word,textArray[j])
                                //console.log(wordTypes[i],textArray[j])
                                stop = true;
                            }else{
                                wordNumb += textArray[j].length +1;
                            }
                        }
                    }
                    textFormats.push({color:wordTypes[i].color, wordNumb:wordNumb});
                    textFormats.push({color:style.fill, wordNumb:wordNumb+wordTypes[i].word.length});
                }
                ///Чистим цвета теста
                that.texts[num - 1].clearColors();
                for(let i in textFormats){
                    ///Добавляем цвета текста
                    that.texts[num - 1].addColor(textFormats[i].color, textFormats[i].wordNumb);
                    ///addFontStyle (стиль, позиция) ///normal, italicили oblique
                    ///addFontWeight (стиль, позиция) ///normal, bold, bolder, lighter
                }
                lastTextString = that.texts[num - 1].text;
            }
            
        });
        return this.texts[num - 1];
    }
    */
    }